@@ -223,6 +223,13 @@ void BreakpointController::handleBreakpointListInitial(const GDBMI::ResultRecord
                     updateBreakpoint = b;
                 }
             } else if (b->kind() == KDevelop::Breakpoint::CodeBreakpoint) {
+                QString condition;
+                if (mi_b.hasField("cond")) {
+                    condition = mi_b["cond"].literal();
+                }
+                if (condition != b->condition())
+                    continue;
+
                 QString location = mi_b["original-location"].literal();
                 kDebug() << "location" << location;
                 QRegExp rx("^(.+):(\\d+)$");
@@ -232,6 +239,8 @@ void BreakpointController::handleBreakpointListInitial(const GDBMI::ResultRecord
                     } else {
                         kDebug() << "!=" << b->location();
                     }
+                } else if (location == b->location()) {
+                    updateBreakpoint = b;
                 }
             }
             if (updateBreakpoint) break;
@@ -401,7 +410,12 @@ void BreakpointController::handleBreakpointList(const GDBMI::ResultRecord &r)
             } else if (type == "acc watchpoint") {
                 b = breakpointModel()->addAccessWatchpoint();
             } else {
-                b = breakpointModel()->addCodeBreakpoint();
+                //for multiple breakpoints(constructor/destructor... ) show only the parent breakpoint(1, 1.1, 1.2, ... only 1), because all other can be usefull only in dissasembleWidget.
+                if(mi_b.hasField("number") && !mi_b["number"].literal().contains(".")){
+                    b = breakpointModel()->addCodeBreakpoint();
+                }else{
+                    continue;
+                }
             }
             previousType = type;
         }
@@ -435,7 +449,12 @@ void BreakpointController::update(KDevelop::Breakpoint *breakpoint, const GDBMI:
                 if (rx.indexIn(location) != -1) {
                     breakpoint->setLocation(KUrl(unquoteExpression(rx.cap(1))), rx.cap(2).toInt()-1);
                 } else {
-                    kWarning() << "can't parse location" << location;
+                    //for regular expression breakpoints and not only...
+                    if(b.hasField("fullname") && b.hasField("line")){
+                        breakpoint->setLocation(KUrl(unquoteExpression(b["fullname"].literal())), b["line"].toInt()-1);
+                    }else{
+                        kWarning() << "can't parse location" << location;
+                    }
                 }
             } else {
                 breakpoint->setData(KDevelop::Breakpoint::LocationColumn, unquoteExpression(location));
