@@ -90,7 +90,7 @@ namespace GDBDebugger
 {
 
 K_PLUGIN_FACTORY(CppDebuggerFactory, registerPlugin<CppDebuggerPlugin>(); )
-K_EXPORT_PLUGIN(CppDebuggerFactory(KAboutData("kdevgdb","kdevgdb", ki18n("GDB Support"), "0.1", ki18n("Support for running apps in GDB"), KAboutData::License_GPL)))
+K_EXPORT_PLUGIN(CppDebuggerFactory(KAboutData("kdevgdb","kdevgdb", ki18n("GDB Support"), "0.1", ki18n("This plugin provides a frontend for GDB, a source-level debugger for C, C++ and more."), KAboutData::License_GPL)))
 
 template<class T>
 class DebuggerToolFactory : public KDevelop::IToolViewFactory
@@ -117,7 +117,7 @@ public:
 
   virtual void viewCreated(Sublime::View* view)
   {
-      if (view->widget()->metaObject()->indexOfSignal(SIGNAL(requestRaise())) != -1)
+      if (view->widget()->metaObject()->indexOfSignal(QMetaObject::normalizedSignature("requestRaise()")) != -1)
           QObject::connect(view->widget(), SIGNAL(requestRaise()), view, SLOT(requestRaise()));
   }
 
@@ -152,31 +152,31 @@ CppDebuggerPlugin::CppDebuggerPlugin( QObject *parent, const QVariantList & ) :
     this, "org.kdevelop.debugger.ConsoleView",Qt::BottomDockWidgetArea);
 
     core()->uiController()->addToolView(
-        i18n("Disassemble"),
+        i18n("Disassemble/Registers"),
         disassemblefactory);
 
     core()->uiController()->addToolView(
         i18n("GDB"),
         gdbfactory);
 
-    /*
-    viewerfactory = new DebuggerToolFactory<ViewerWidget>(
-    this, "org.kdevelop.debugger.VariousViews", Qt::BottomDockWidgetArea);
+    memoryviewerfactory = new DebuggerToolFactory<MemoryViewerWidget>(
+    this, "org.kdevelop.debugger.MemoryView", Qt::BottomDockWidgetArea);
     core()->uiController()->addToolView(
-        i18n("Debug views"),
-        viewerfactory);
-    */
+        i18n("Memory"),
+        memoryviewerfactory);
 
     setupActions();
 
     setupDBus();
 
-    IExecutePlugin* iface = KDevelop::ICore::self()->pluginController()->pluginForExtension("org.kdevelop.IExecutePlugin")->extension<IExecutePlugin>();
-    Q_ASSERT(iface);
-    KDevelop::LaunchConfigurationType* type = core()->runController()->launchConfigurationTypeForId( iface->nativeAppConfigTypeId() );
-    Q_ASSERT(type);
-    type->addLauncher( new GdbLauncher( this ) );
-    
+    QList<IPlugin*> plugins = KDevelop::ICore::self()->pluginController()->allPluginsForExtension("org.kdevelop.IExecutePlugin");
+    foreach(IPlugin* plugin, plugins) {
+        IExecutePlugin* iface = plugin->extension<IExecutePlugin>();
+        Q_ASSERT(iface);
+        KDevelop::LaunchConfigurationType* type = core()->runController()->launchConfigurationTypeForId( iface->nativeAppConfigTypeId() );
+        Q_ASSERT(type);
+        type->addLauncher( new GdbLauncher( this, iface ) );
+    }
     // The output from tracepoints goes to "application" window, because
     // we don't have any better alternative, and using yet another window
     // is undesirable. Besides, this makes tracepoint look even more similar
@@ -190,26 +190,13 @@ void CppDebuggerPlugin::unload()
 {
     core()->uiController()->removeToolView(disassemblefactory);
     core()->uiController()->removeToolView(gdbfactory);
-    //core()->uiController()->removeToolView(viewerfactory);
+    core()->uiController()->removeToolView(memoryviewerfactory);
 }
 
 void CppDebuggerPlugin::setupActions()
 {
     KActionCollection* ac = actionCollection();
 
-    /*
-    KAction* action = new KAction(KIcon("dbgmemview"), i18n("Viewers"), this);
-    action->setToolTip( i18n("Debugger viewers") );
-    action->setWhatsThis(i18n("<b>Debugger viewers</b><p>Various information about application being executed. There are 4 views available:<br>"
-        "<b>Memory</b><br>"
-        "<b>Disassemble</b><br>"
-        "<b>Registers</b><br>"
-        "<b>Libraries</b>"));
-    connect(action, SIGNAL(triggered(bool)), this, SIGNAL(addMemoryView()));
-    ac->addAction("debug_memview", action);
-    */
-
-
     KAction* action = new KAction(KIcon("core"), i18n("Examine Core File..."), this);
     action->setToolTip( i18n("Examine core file") );
     action->setWhatsThis( i18n("<b>Examine core file</b><p>"
@@ -360,8 +347,7 @@ DebugSession* CppDebuggerPlugin::createSession()
     connect(session, SIGNAL(showMessage(QString,int)), SLOT(controllerMessage(QString,int)));
     connect(session, SIGNAL(reset()), SIGNAL(reset()));
     connect(session, SIGNAL(finished()), SLOT(slotFinished()));
-    connect(session, SIGNAL(raiseOutputViews()), SIGNAL(raiseOutputViews()));
-    connect(session, SIGNAL(raiseVariableViews()), SIGNAL(raiseVariableViews()));
+    connect(session, SIGNAL(raiseGdbConsoleViews()), SIGNAL(raiseGdbConsoleViews()));
     return session;
 }
 
