@@ -27,6 +27,7 @@
 #define GDB_DEBUGSESSION_H
 
 #include <QWeakPointer>
+#include <QScopedPointer>
 
 #include <KConfigGroup>
 
@@ -35,6 +36,7 @@
 #include "gdbglobal.h"
 #include "mi/gdbmi.h"
 
+class IExecutePlugin;
 class KToolBar;
 
 namespace KTextEditor {
@@ -87,7 +89,7 @@ public Q_SLOTS:
     /**
      * Start the debugger, and execute the program specified by \a run.
      */
-    bool startProgram(KDevelop::ILaunchConfiguration* run);
+    bool startProgram(KDevelop::ILaunchConfiguration* run, IExecutePlugin* execute);
     virtual void restartDebugger();
     virtual void stopDebugger();
     virtual void interruptDebugger();
@@ -139,11 +141,20 @@ public:
      */
     void runUntil(const KUrl& url, int line);
 
+    /**
+     * Run currently executing program to the given \a address
+     */
+    void runUntil(QString& address);
     /**
      * Move the execution point of the currently executing program to the given \a url and \a line.
      */
     void jumpTo(const KUrl& url, int line);
 
+    /**
+     * Move the execution point of the currently executing program to the given \a address.
+     *Note: It can be really very dangerous, so use jumpTo instead.
+     */
+    void jumpToMemoryAddress(QString& address);
 
     /** Adds a command to the end of queue of commands to be executed
         by gdb. The command will be actually sent to gdb only when
@@ -211,7 +222,7 @@ private:
     void destroyCmds();
     void removeInfoRequests();
     /** Called when there are no pending commands and 'state_reload_needed'
-        is true.
+        is true. Also can be used to immediately reload program state.
         Issues commands to completely reload all program state shown
         to the user.
     */
@@ -233,6 +244,9 @@ private:
 
     bool startDebugger(KDevelop::ILaunchConfiguration* cfg);
 
+    ///Checks if exited inferior is the last one, if so ends the debug session.
+    void lastInferiorHandler(const QStringList& l);
+
 private Q_SLOTS:
 
     void gdbReady();
@@ -256,6 +270,7 @@ private Q_SLOTS:
         Otherwise, shows a dialog box and reloads view state.  */
     void defaultErrorHandler(const GDBMI::ResultRecord& result);
 
+    /**Triggered every time program begins/continues it's execution.*/
     void programRunning();
 
     // All of these slots are entered in the controller's thread, as they use queued connections or are called internally
@@ -304,19 +319,22 @@ private:
 
     CommandQueue*   commandQueue_;
 
-    STTY*             tty_;
+    QScopedPointer<STTY> m_tty;
     QString           badCore_;
 
     // Some state variables
     DBGStateFlags     state_;
-    bool              programHasExited_;
-
 
+    /**When program stops and all commands from queue are executed and this variable is true, program state shown to the user is updated.*/
     bool state_reload_needed;
 
     QTime commandExecutionTime;
 
+    /**True if program has stopped and all stuff like breakpoints is being updated.*/
     bool stateReloadInProgress_;
+
+    ///Exit code of the last inferior(in format: exit normally, with code "number" e.t.c)
+    QString m_inferiorExitCode;
 };
 
 }
