@@ -1,5 +1,6 @@
 /*
    Copyright 2009 Niko Sams <niko.sams@gmail.com>
+   Copyright 2013 Vlas Puhov <vlas.puhov@mail.ru>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -29,6 +30,7 @@
 #include <KSharedConfig>
 #include <KDebug>
 #include <KProcess>
+#include <KStandardDirs>
 #include <qtest_kde.h>
 
 #include <tests/testcore.h>
@@ -38,10 +40,12 @@
 #include <debugger/breakpoint/breakpoint.h>
 #include <debugger/interfaces/ibreakpointcontroller.h>
 #include <interfaces/ilaunchconfiguration.h>
+#include <interfaces/iplugincontroller.h>
 #include <debugger/variable/variablecollection.h>
 #include <debugger/interfaces/ivariablecontroller.h>
 #include <debugger/framestack/framestackmodel.h>
 #include <tests/autotestshell.h>
+#include <execute/iexecuteplugin.h>
 
 #include "gdbcommand.h"
 #include "debugsession.h"
@@ -72,6 +76,9 @@ void GdbTest::initTestCase()
 {
     AutoTestShell::init();
     KDevelop::TestCore::initialize(KDevelop::Core::NoUi);
+
+    m_iface = KDevelop::ICore::self()->pluginController()->pluginForExtension("org.kdevelop.IExecutePlugin", "kdevexecute")->extension<IExecutePlugin>();
+    Q_ASSERT(m_iface);
 }
 
 void GdbTest::cleanupTestCase()
@@ -196,7 +203,7 @@ void GdbTest::testStdOut()
     QSignalSpy outputSpy(session, SIGNAL(applicationStandardOutputLines(QStringList)));
 
     TestLaunchConfiguration cfg;
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
     WAIT_FOR_STATE(session, KDevelop::IDebugSession::EndedState);
 
     {
@@ -216,7 +223,7 @@ void GdbTest::testBreakpoint()
     KDevelop::Breakpoint * b = breakpoints()->addCodeBreakpoint(debugeeFileName, 28);
     QCOMPARE(session->breakpointController()->breakpointState(b), KDevelop::Breakpoint::NotStartedState);
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QCOMPARE(session->breakpointController()->breakpointState(b), KDevelop::Breakpoint::CleanState);
     session->stepInto();
@@ -229,32 +236,49 @@ void GdbTest::testBreakpoint()
 
 void GdbTest::testDisableBreakpoint()
 {
+    //Description: We must stop only on the third breakpoint
+
+    int firstBreakLine=28;
+    int secondBreakLine=23;
+    int thirdBreakLine=24;
+    int fourthBreakLine=31;
+
     TestDebugSession *session = new TestDebugSession;
 
     TestLaunchConfiguration cfg;
 
     KDevelop::Breakpoint *b;
 
-    //add disabled breakpoint before startProgram
-    b = breakpoints()->addCodeBreakpoint(debugeeFileName, 29);
-    b->setData(KDevelop::Breakpoint::EnableColumn, false);
+    b = breakpoints()->addCodeBreakpoint(debugeeFileName, firstBreakLine);
+    b->setData(KDevelop::Breakpoint::EnableColumn, Qt::Unchecked);
 
-    b = breakpoints()->addCodeBreakpoint(debugeeFileName, 21);
-    session->startProgram(&cfg);
+
+    //this is needed to emulate debug from GUI. If we are in edit mode, the debugSession doesn't exist.
+    KDevelop::ICore::self()->debugController()->breakpointModel()->blockSignals(true);
+    b = breakpoints()->addCodeBreakpoint(debugeeFileName, secondBreakLine);
+    b->setData(KDevelop::Breakpoint::EnableColumn, Qt::Unchecked);
+    //all disabled breakpoints were added
+
+    KDevelop::Breakpoint * thirdBreak = breakpoints()->addCodeBreakpoint(debugeeFileName, thirdBreakLine);
+    KDevelop::ICore::self()->debugController()->breakpointModel()->blockSignals(false);
+
+
+    session->startProgram(&cfg, m_iface);
     WAIT_FOR_STATE(session, DebugSession::PausedState);
 
+    QCOMPARE(session->currentLine(), thirdBreak->line());
+
     //disable existing breakpoint
-    b->setData(KDevelop::Breakpoint::EnableColumn, false);
+    thirdBreak->setData(KDevelop::Breakpoint::EnableColumn, Qt::Unchecked);
 
     //add another disabled breakpoint
-    b = breakpoints()->addCodeBreakpoint(debugeeFileName, 31);
+    b = breakpoints()->addCodeBreakpoint(debugeeFileName, fourthBreakLine);
     QTest::qWait(300);
-    b->setData(KDevelop::Breakpoint::EnableColumn, false);
+    b->setData(KDevelop::Breakpoint::EnableColumn, Qt::Unchecked);
 
     QTest::qWait(300);
     session->run();
     WAIT_FOR_STATE(session, DebugSession::EndedState);
-
 }
 
 void GdbTest::testChangeLocationBreakpoint()
@@ -265,7 +289,7 @@ void GdbTest::testChangeLocationBreakpoint()
 
     KDevelop::Breakpoint *b = breakpoints()->addCodeBreakpoint(debugeeFileName, 27);
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QCOMPARE(session->line(), 27);
 
@@ -306,7 +330,7 @@ void GdbTest::testDeleteBreakpoint()
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 22);
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     breakpoints()->removeRow(0);
     QTest::qWait(100);
@@ -325,7 +349,7 @@ void GdbTest::testPendingBreakpoint()
     KDevelop::Breakpoint * b = breakpoints()->addCodeBreakpoint(findSourceFile("/gdbtest.cpp"), 10);
     QCOMPARE(session->breakpointController()->breakpointState(b), KDevelop::Breakpoint::NotStartedState);
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QCOMPARE(session->breakpointController()->breakpointState(b), KDevelop::Breakpoint::PendingState);
     session->run();
@@ -340,7 +364,7 @@ void GdbTest::testUpdateBreakpoint()
     KDevelop::Breakpoint * b = breakpoints()->addCodeBreakpoint(debugeeFileName, 28);
     QCOMPARE(KDevelop::ICore::self()->debugController()->breakpointModel()->rowCount(), 2);
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
 
     //insert custom command as user might do it using GDB console
     session->addCommand(new UserCommand(GDBMI::NonMI, "break "+debugeeFileName+":28"));
@@ -367,7 +391,7 @@ void GdbTest::testIgnoreHitsBreakpoint()
 
     b = breakpoints()->addCodeBreakpoint(debugeeFileName, 22);
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
 
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QTest::qWait(100);
@@ -391,7 +415,7 @@ void GdbTest::testConditionBreakpoint()
 
     b = breakpoints()->addCodeBreakpoint(debugeeFileName, 24);
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
 
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QCOMPARE(session->line(), 24);
@@ -414,7 +438,7 @@ void GdbTest::testBreakOnWriteBreakpoint()
     
     breakpoints()->addCodeBreakpoint(debugeeFileName, 24);
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
 
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QCOMPARE(session->line(), 24);
@@ -439,7 +463,7 @@ void GdbTest::testBreakOnWriteWithConditionBreakpoint()
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 24);
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
 
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QCOMPARE(session->line(), 24);
@@ -468,7 +492,7 @@ void GdbTest::testBreakOnReadBreakpoint()
 
     KDevelop::Breakpoint *b = breakpoints()->addReadWatchpoint("foo::i");
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
 
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QCOMPARE(session->line(), 23);
@@ -484,7 +508,7 @@ void GdbTest::testBreakOnReadBreakpoint2()
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 24);
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
 
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QCOMPARE(session->line(), 24);
@@ -510,7 +534,7 @@ void GdbTest::testBreakOnAccessBreakpoint()
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 24);
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
 
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QCOMPARE(session->line(), 24);
@@ -540,7 +564,7 @@ void GdbTest::testInsertBreakpointWhileRunning()
     TestLaunchConfiguration cfg(findExecutable("debugeeslow"));
     QString fileName = findSourceFile("debugeeslow.cpp");
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
 
     WAIT_FOR_STATE(session, DebugSession::ActiveState);
     QTest::qWait(2000);
@@ -560,7 +584,7 @@ void GdbTest::testInsertBreakpointWhileRunningMultiple()
     TestLaunchConfiguration cfg(findExecutable("debugeeslow"));
     QString fileName = findSourceFile("debugeeslow.cpp");
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
 
     WAIT_FOR_STATE(session, DebugSession::ActiveState);
     QTest::qWait(2000);
@@ -586,7 +610,7 @@ void GdbTest::testInsertBreakpointFunctionName()
 
     breakpoints()->addCodeBreakpoint("main");
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QCOMPARE(session->line(), 27);
     session->run();
@@ -603,7 +627,7 @@ void GdbTest::testShowStepInSource()
     TestLaunchConfiguration cfg;
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 29);
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     session->stepInto();
     WAIT_FOR_STATE(session, DebugSession::PausedState);
@@ -636,7 +660,7 @@ void GdbTest::testStack()
     TestFrameStackModel *stackModel = session->frameStackModel();
     
     breakpoints()->addCodeBreakpoint(debugeeFileName, 21);
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     WAIT_FOR_STATE(session, DebugSession::PausedState);
 
     QModelIndex tIdx = stackModel->index(0,0);    
@@ -678,7 +702,7 @@ void GdbTest::testStackFetchMore()
     TestFrameStackModel *stackModel = session->frameStackModel();
 
     breakpoints()->addCodeBreakpoint(fileName, 25);
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QCOMPARE(session->frameStackModel()->fetchFramesCalled, 1);
 
@@ -713,29 +737,28 @@ void GdbTest::testStackFetchMore()
     stackModel->fetchMoreFrames();
     QTest::qWait(200);
     QCOMPARE(stackModel->fetchFramesCalled, 3);
-    QCOMPARE(stackModel->rowCount(tIdx), 61);
+    QCOMPARE(stackModel->rowCount(tIdx), 121);
     COMPARE_DATA(tIdx.child(40, 0), "40");
     COMPARE_DATA(tIdx.child(41, 0), "41");
     COMPARE_DATA(tIdx.child(42, 0), "42");
-    COMPARE_DATA(tIdx.child(60, 0), "60");
+    COMPARE_DATA(tIdx.child(119, 0), "119");
+    COMPARE_DATA(tIdx.child(120, 0), "120");
 
     stackModel->fetchMoreFrames();
     QTest::qWait(200);
     QCOMPARE(stackModel->fetchFramesCalled, 4);
-    QCOMPARE(stackModel->rowCount(tIdx), 81);
-
-    stackModel->fetchMoreFrames();
-    QTest::qWait(200);
-    QCOMPARE(stackModel->fetchFramesCalled, 5);
-    QCOMPARE(stackModel->rowCount(tIdx), 101);
-    COMPARE_DATA(tIdx.child(100, 0), "100");
-    COMPARE_DATA(tIdx.child(100, 1), "main");
-    COMPARE_DATA(tIdx.child(100, 2), fileName+":30");
+    QCOMPARE(stackModel->rowCount(tIdx), 301);
+    COMPARE_DATA(tIdx.child(120, 0), "120");
+    COMPARE_DATA(tIdx.child(121, 0), "121");
+    COMPARE_DATA(tIdx.child(122, 0), "122");
+    COMPARE_DATA(tIdx.child(300, 0), "300");
+    COMPARE_DATA(tIdx.child(300, 1), "main");
+    COMPARE_DATA(tIdx.child(300, 2), fileName+":30");
 
     stackModel->fetchMoreFrames(); //nothing to fetch, we are at the end
     QTest::qWait(200);
-    QCOMPARE(stackModel->fetchFramesCalled, 5);
-    QCOMPARE(stackModel->rowCount(tIdx), 101);
+    QCOMPARE(stackModel->fetchFramesCalled, 4);
+    QCOMPARE(stackModel->rowCount(tIdx), 301);
 
     session->run();
     WAIT_FOR_STATE(session, DebugSession::EndedState);
@@ -749,7 +772,7 @@ void GdbTest::testStackDeactivateAndActive()
     TestFrameStackModel *stackModel = session->frameStackModel();
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 21);
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     WAIT_FOR_STATE(session, DebugSession::PausedState);
 
     QModelIndex tIdx = stackModel->index(0,0);
@@ -778,7 +801,7 @@ void GdbTest::testStackSwitchThread()
     TestFrameStackModel *stackModel = session->frameStackModel();
 
     breakpoints()->addCodeBreakpoint(fileName, 38);
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     QTest::qWait(500);
     WAIT_FOR_STATE(session, DebugSession::PausedState);
 
@@ -804,6 +827,15 @@ void GdbTest::testStackSwitchThread()
 
 void GdbTest::testAttach()
 {
+    // if on linux, ensure we can actually attach
+    QFile canRun("/proc/sys/kernel/yama/ptrace_scope");
+    if (canRun.exists()) {
+        QVERIFY(canRun.open(QIODevice::ReadOnly));
+        if (canRun.read(1).toInt() != 0) {
+            QSKIP("ptrace attaching not allows, skipping test. To enable it, set /proc/sys/kernel/yama/ptrace_scope to 0.", SkipAll);
+        }
+    }
+
     QString fileName = findSourceFile("debugeeslow.cpp");
 
     KProcess debugeeProcess;
@@ -821,7 +853,9 @@ void GdbTest::testAttach()
     session->run();
     QTest::qWait(2000);
     WAIT_FOR_STATE(session, DebugSession::PausedState);
-    QCOMPARE(session->line(), 34);
+    if (session->line() < 34 || session->line() < 35) {
+        QCOMPARE(session->line(), 34);
+    }
 
     session->run();
     WAIT_FOR_STATE(session, DebugSession::EndedState);
@@ -872,7 +906,7 @@ void GdbTest::testVariablesLocals()
     TestLaunchConfiguration cfg;
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 22);
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QTest::qWait(1000);
 
@@ -899,7 +933,7 @@ void GdbTest::testVariablesLocalsStruct()
     TestLaunchConfiguration cfg;
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 38);
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QTest::qWait(1000);
 
@@ -945,7 +979,7 @@ void GdbTest::testVariablesWatches()
     TestLaunchConfiguration cfg;
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 38);
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     
     variableCollection()->watches()->add("ts");
@@ -988,7 +1022,7 @@ void GdbTest::testVariablesWatchesQuotes()
     const QString quotedTestString("\"" + testString + "\"");
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 38);
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     WAIT_FOR_STATE(session, DebugSession::PausedState);
 
     variableCollection()->watches()->add(quotedTestString); //just a constant string
@@ -1026,7 +1060,7 @@ void GdbTest::testVariablesWatchesTwoSessions()
     TestLaunchConfiguration cfg;
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 38);
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     
     variableCollection()->watches()->add("ts");
@@ -1050,7 +1084,7 @@ void GdbTest::testVariablesWatchesTwoSessions()
     //start a second debug session
     session = new TestDebugSession;
     session->variableController()->setAutoUpdate(KDevelop::IVariableController::UpdateWatches);
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QTest::qWait(300);
 
@@ -1082,7 +1116,7 @@ void GdbTest::testVariablesStopDebugger()
     TestLaunchConfiguration cfg;
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 38);
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     WAIT_FOR_STATE(session, DebugSession::PausedState);
 
     session->stopDebugger();
@@ -1098,14 +1132,14 @@ void GdbTest::testVariablesStartSecondSession()
     TestLaunchConfiguration cfg;
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 38);
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     WAIT_FOR_STATE(session, DebugSession::PausedState);
 
     session = new TestDebugSession;
     session->variableController()->setAutoUpdate(KDevelop::IVariableController::UpdateLocals);
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 38);
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     WAIT_FOR_STATE(session, DebugSession::PausedState);
 
     session->run();
@@ -1121,7 +1155,7 @@ void GdbTest::testVariablesSwitchFrame()
     TestFrameStackModel *stackModel = session->frameStackModel();
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 24);
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QTest::qWait(500);
 
@@ -1154,7 +1188,7 @@ void GdbTest::testVariablesQuicklySwitchFrame()
     TestFrameStackModel *stackModel = session->frameStackModel();
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 24);
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QTest::qWait(500);
 
@@ -1200,7 +1234,7 @@ void GdbTest::testSegfaultDebugee()
 
     breakpoints()->addCodeBreakpoint(fileName, 23);
 
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
 
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QCOMPARE(session->line(), 23);
@@ -1222,7 +1256,7 @@ void GdbTest::testSwitchFrameGdbConsole()
     TestFrameStackModel *stackModel = session->frameStackModel();
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 24);
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QCOMPARE(stackModel->currentFrame(), 0);
     stackModel->setCurrentFrame(1);
@@ -1244,7 +1278,7 @@ void GdbTest::testInsertAndRemoveBreakpointWhileRunning()
     TestLaunchConfiguration cfg(findExecutable("debugeeslow"));
     QString fileName = findSourceFile("debugeeslow.cpp");
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
 
     WAIT_FOR_STATE(session, DebugSession::ActiveState);
     QTest::qWait(2000);
@@ -1262,7 +1296,7 @@ void GdbTest::testCommandOrderFastStepping()
     TestLaunchConfiguration cfg(findExecutable("debugeeqt"));
 
     breakpoints()->addCodeBreakpoint("main");
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     for(int i=0; i<20; i++) {
         session->stepInto();
     }
@@ -1278,7 +1312,7 @@ void GdbTest::testPickupManuallyInsertedBreakpoint()
     TestLaunchConfiguration cfg;
 
     breakpoints()->addCodeBreakpoint("main");
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     session->addCommand(GDBMI::NonMI, "break debugee.cpp:32");
     session->stepInto();
     WAIT_FOR_STATE(session, DebugSession::PausedState);
@@ -1299,23 +1333,29 @@ void GdbTest::testPickupManuallyInsertedBreakpointOnlyOnce()
     TestLaunchConfiguration cfg;
 
     breakpoints()->addCodeBreakpoint(KUrl("debugee.cpp"), 31);
-    QVERIFY(session->startProgram(&cfg));
+    breakpoints()->addCodeBreakpoint(KUrl("debugee.cpp"), 21);
+    QVERIFY(session->startProgram(&cfg, m_iface));
 
     //inject here, so it behaves similar like a command from .gdbinit
     session->addCommandToFront(new GDBCommand(GDBMI::NonMI, "break debugee.cpp:32"));
+    session->addCommandToFront(new GDBCommand(GDBMI::NonMI, "break foo"));
 
     WAIT_FOR_STATE(session, DebugSession::PausedState);
 
     session->stepInto();
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QTest::qWait(1000); //wait for breakpoints update
-    QCOMPARE(breakpoints()->breakpoints().count(), 1);
-    QCOMPARE(breakpoints()->rowCount(), 1+1);
+    QCOMPARE(breakpoints()->breakpoints().count(), 2);
 
     KDevelop::Breakpoint *b = breakpoints()->breakpoint(0);
     QVERIFY(b);
     QCOMPARE(b->line(), 31); //we start with 0, gdb with 1
     QCOMPARE(b->url().url(), QString("debugee.cpp"));
+
+    b = breakpoints()->breakpoint(1);
+    QVERIFY(b);
+    QCOMPARE(b->line(), 21);
+    QCOMPARE(b->url().url(), QString("debugee.cpp"));
 }
 
 void GdbTest::testRunGdbScript()
@@ -1334,7 +1374,7 @@ void GdbTest::testRunGdbScript()
     KConfigGroup grp = cfg.config();
     grp.writeEntry(GDBDebugger::remoteGdbRunEntry, KUrl(runScript.fileName()));
 
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
 
     WAIT_FOR_STATE(session, DebugSession::PausedState);
 
@@ -1346,6 +1386,11 @@ void GdbTest::testRunGdbScript()
 
 void GdbTest::testRemoteDebug()
 {
+    const QString gdbserverExecutable = KStandardDirs::findExe("gdbserver");
+    if (gdbserverExecutable.isEmpty()) {
+        QSKIP("Skipping, gdbserver not available", SkipSingle);
+    }
+
     TestDebugSession *session = new TestDebugSession;
 
     QTemporaryFile shellScript(QDir::currentPath()+"/shellscript");
@@ -1368,7 +1413,7 @@ void GdbTest::testRemoteDebug()
     grp.writeEntry(GDBDebugger::remoteGdbShellEntry, KUrl(shellScript.fileName()+"-copy"));
     grp.writeEntry(GDBDebugger::remoteGdbRunEntry, KUrl(runScript.fileName()));
 
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
 
     WAIT_FOR_STATE(session, DebugSession::PausedState);
 
@@ -1382,6 +1427,11 @@ void GdbTest::testRemoteDebug()
 
 void GdbTest::testRemoteDebugInsertBreakpoint()
 {
+    const QString gdbserverExecutable = KStandardDirs::findExe("gdbserver");
+    if (gdbserverExecutable.isEmpty()) {
+        QSKIP("Skipping, gdbserver not available", SkipSingle);
+    }
+
     TestDebugSession *session = new TestDebugSession;
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 35);
@@ -1406,7 +1456,7 @@ void GdbTest::testRemoteDebugInsertBreakpoint()
     grp.writeEntry(GDBDebugger::remoteGdbShellEntry, KUrl(shellScript.fileName()+"-copy"));
     grp.writeEntry(GDBDebugger::remoteGdbRunEntry, KUrl(runScript.fileName()));
 
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
 
     WAIT_FOR_STATE(session, DebugSession::PausedState);
 
@@ -1428,6 +1478,11 @@ void GdbTest::testRemoteDebugInsertBreakpoint()
 
 void GdbTest::testRemoteDebugInsertBreakpointPickupOnlyOnce()
 {
+    const QString gdbserverExecutable = KStandardDirs::findExe("gdbserver");
+    if (gdbserverExecutable.isEmpty()) {
+        QSKIP("Skipping, gdbserver not available", SkipSingle);
+    }
+
     TestDebugSession *session = new TestDebugSession;
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 35);
@@ -1452,7 +1507,7 @@ void GdbTest::testRemoteDebugInsertBreakpointPickupOnlyOnce()
     grp.writeEntry(GDBDebugger::remoteGdbShellEntry, KUrl(shellScript.fileName()+"-copy"));
     grp.writeEntry(GDBDebugger::remoteGdbRunEntry, KUrl(runScript.fileName()));
 
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
 
     WAIT_FOR_STATE(session, DebugSession::PausedState);
 
@@ -1470,7 +1525,7 @@ void GdbTest::testRemoteDebugInsertBreakpointPickupOnlyOnce()
 
     //************************** second session
     session = new TestDebugSession;
-    QVERIFY(session->startProgram(&cfg));
+    QVERIFY(session->startProgram(&cfg, m_iface));
 
     WAIT_FOR_STATE(session, DebugSession::PausedState);
 
@@ -1500,7 +1555,7 @@ void GdbTest::testBreakpointWithSpaceInPath()
     KDevelop::Breakpoint * b = breakpoints()->addCodeBreakpoint(fileName, 20);
     QCOMPARE(session->breakpointController()->breakpointState(b), KDevelop::Breakpoint::NotStartedState);
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
     WAIT_FOR_STATE_FAIL(session, DebugSession::PausedState);
     QCOMPARE(session->line(), 20);
     session->run();
@@ -1519,7 +1574,7 @@ void GdbTest::testBreakpointDisabledOnStart()
     KDevelop::Breakpoint* b = breakpoints()->addCodeBreakpoint(debugeeFileName, 31);
     b->setData(KDevelop::Breakpoint::EnableColumn, Qt::Unchecked);
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QCOMPARE(session->line(), 29);
     b->setData(KDevelop::Breakpoint::EnableColumn, Qt::Checked);
@@ -1539,52 +1594,56 @@ void GdbTest::testCatchpoint()
 
     breakpoints()->addCodeBreakpoint(findSourceFile("debugeeexception.cpp"), 29);
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QTest::qWait(1000);
     TestFrameStackModel* fsModel = session->frameStackModel();
     QCOMPARE(fsModel->currentFrame(), 0);
     QCOMPARE(session->line(), 29);
 
-    QModelIndex i = variableCollection()->index(1, 0);
-    COMPARE_DATA(i, "Locals");
-    qDebug() << variableCollection()->index(0, 1, i);
-    qDebug() << variableCollection()->index(1, 1, i);
-    QCOMPARE(variableCollection()->rowCount(i), 2);
-
     session->addCommand(new GDBCommand(GDBMI::NonMI, "catch throw"));
     session->run();
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QTest::qWait(1000);
-    QCOMPARE(fsModel->currentFrame(), 1); //first frame skiped because somewhere inside stdlib
-    QCOMPARE(session->line(), 22);
 
-    qDebug() << variableCollection()->index(0, 1, i);
-    qDebug() << variableCollection()->index(1, 1, i);
-    QCOMPARE(variableCollection()->rowCount(i), 1);
-    COMPARE_DATA(variableCollection()->index(0, 0, i), "i");
-    COMPARE_DATA(variableCollection()->index(0, 1, i), "20");
+    // TODO: Fix API in FrameStackModel? At least introduce FrameStackModel::frames(int)
+    typedef KDevelop::IFrameStackModel::FrameItem FrameItem;
+    QVector<FrameItem> frames;
+    const QModelIndex thread1Index = fsModel->index(0, 0);
+    for (int i = 0; i < fsModel->rowCount(thread1Index); ++i) {
+        FrameItem frame = fsModel->frame(fsModel->index(i, 0, thread1Index));
+        frames << frame;
+        qDebug() << frame.file << frame.line;
+    }
+
+    QVERIFY(frames.size() >= 2);
+    // frame 0 is somewhere inside libstdc++
+    QCOMPARE(frames[1].file, KUrl(findSourceFile("debugeeexception.cpp")));
+    QCOMPARE(frames[1].line, 22);
 
     session->run();
     WAIT_FOR_STATE(session, DebugSession::EndedState);
-
 }
 
+//TODO: figure out why do we need this test? And do we need it at all??
 void GdbTest::testThreadAndFrameInfo()
 {
     TestDebugSession *session = new TestDebugSession;
     TestLaunchConfiguration cfg(findExecutable("debugeethreads"));
     QString fileName = findSourceFile("debugeethreads.cpp");
-    QSignalSpy outputSpy(session, SIGNAL(gdbUserCommandStdout(QString)));
 
     breakpoints()->addCodeBreakpoint(fileName, 38);
-    QVERIFY(session->startProgram(&cfg));
-    session->frameStackModel()->fetchThreads();
-    session->addCommand(new UserCommand(GDBMI::StackListLocals, QLatin1String("0")));
+    QVERIFY(session->startProgram(&cfg, m_iface));
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QTest::qWait(1000);
 
-    QVERIFY(outputSpy.count() == 2);
+    QSignalSpy outputSpy(session, SIGNAL(gdbUserCommandStdout(QString)));
+
+    session->addCommand(
+                new UserCommand(GDBMI::ThreadInfo,""));
+    session->addCommand(new UserCommand(GDBMI::StackListLocals, QLatin1String("0")));
+    QTest::qWait(1000);
+    QCOMPARE(outputSpy.count(), 2);
     QVERIFY(outputSpy.last().at(0).toString().contains(QLatin1String("--thread 1")));
 
     session->run();
@@ -1629,7 +1688,7 @@ void GdbTest::testMultipleLocationsBreakpoint()
 
     //TODO check if the additional location breakpoint is added
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
     WAIT_FOR_STATE(session, DebugSession::PausedState);
     QCOMPARE(session->line(), 19);
 
@@ -1650,7 +1709,7 @@ void GdbTest::testBug301287()
 
     breakpoints()->addCodeBreakpoint(debugeeFileName, 28);
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
     WAIT_FOR_STATE(session, DebugSession::PausedState);
 
     variableCollection()->watches()->add("argc");
@@ -1668,7 +1727,7 @@ void GdbTest::testBug301287()
     session = new TestDebugSession;
     session->variableController()->setAutoUpdate(KDevelop::IVariableController::UpdateWatches);
 
-    session->startProgram(&cfg);
+    session->startProgram(&cfg, m_iface);
     WAIT_FOR_STATE(session, DebugSession::PausedState);
 
     QTest::qWait(300);
@@ -1682,6 +1741,95 @@ void GdbTest::testBug301287()
     WAIT_FOR_STATE(session, DebugSession::EndedState);
 }
 
+void GdbTest::testMultipleBreakpoint()
+{
+        TestDebugSession *session = new TestDebugSession;
+
+        //there'll be about 3-4 breakpoints, but we treat it like one.
+        TestLaunchConfiguration c(findExecutable("debugeemultiplebreakpoint"));
+        KDevelop::Breakpoint *b = breakpoints()->addCodeBreakpoint("debugeemultiplebreakpoint.cpp:52");
+        session->startProgram(&c, m_iface);
+        WAIT_FOR_STATE(session, DebugSession::PausedState);
+        QCOMPARE(breakpoints()->breakpoints().count(), 1);
+
+        b->setData(KDevelop::Breakpoint::EnableColumn, Qt::Unchecked);
+        session->run();
+        WAIT_FOR_STATE(session, DebugSession::EndedState);
+}
+
+void GdbTest::testRegularExpressionBreakpoint()
+{
+        TestDebugSession *session = new TestDebugSession;
+
+        TestLaunchConfiguration c(findExecutable("debugeemultilocbreakpoint"));
+        breakpoints()->addCodeBreakpoint("main");
+        session->startProgram(&c, m_iface);
+        WAIT_FOR_STATE(session, DebugSession::PausedState);
+        session->addCommand(new GDBCommand(GDBMI::NonMI, "rbreak .*aPl.*B"));
+        QTest::qWait(100);
+        session->run();
+        WAIT_FOR_STATE(session, DebugSession::PausedState);
+        QCOMPARE(breakpoints()->breakpoints().count(), 3);
+
+        session->addCommand(new GDBCommand(GDBMI::BreakDelete, ""));
+        session->run();
+        WAIT_FOR_STATE(session, DebugSession::EndedState);
+}
+
+void GdbTest::testChangeBreakpointWhileRunning() {
+
+    TestDebugSession *session = new TestDebugSession;
+
+    TestLaunchConfiguration c(findExecutable("debugeeslow"));
+    KDevelop::Breakpoint* b = breakpoints()->addCodeBreakpoint("debugeeslow.cpp:25");
+    session->startProgram(&c, m_iface);
+    
+    WAIT_FOR_STATE(session, DebugSession::PausedState);
+    QVERIFY(session->currentLine() >= 24 && session->currentLine() <= 26 );
+    session->run();
+    WAIT_FOR_STATE(session, DebugSession::ActiveState);
+    b->setData(KDevelop::Breakpoint::EnableColumn, Qt::Unchecked);
+    //to make one loop
+    QTest::qWait(2000);
+    WAIT_FOR_STATE(session, DebugSession::ActiveState);
+    
+    b->setData(KDevelop::Breakpoint::EnableColumn, Qt::Checked);
+    QTest::qWait(100);
+    WAIT_FOR_STATE(session, DebugSession::PausedState);
+    b->setData(KDevelop::Breakpoint::EnableColumn, Qt::Unchecked);
+    session->run();
+    QTest::qWait(100);
+    WAIT_FOR_STATE(session, DebugSession::EndedState);
+}
+
+void GdbTest::testDebugInExternalTerminal()
+{
+    TestLaunchConfiguration cfg;
+
+    foreach (const QString & console, QStringList() << "konsole" << "xterm" << "xfce4-terminal" << "gnome-terminal") {
+
+        TestDebugSession* session = 0;
+        if (KStandardDirs::findExe(console).isEmpty()) {
+            continue;
+        }
+
+        session = new TestDebugSession();
+
+        cfg.config().writeEntry("External Terminal"/*ExecutePlugin::terminalEntry*/, console);
+        cfg.config().writeEntry("Use External Terminal"/*ExecutePlugin::useTerminalEntry*/, true);
+
+        KDevelop::Breakpoint* b = breakpoints()->addCodeBreakpoint(debugeeFileName, 28);
+
+        session->startProgram(&cfg, m_iface);
+        WAIT_FOR_STATE(session, DebugSession::PausedState);
+        QCOMPARE(session->breakpointController()->breakpointState(b), KDevelop::Breakpoint::CleanState);
+        session->stepInto();
+        WAIT_FOR_STATE(session, DebugSession::PausedState);
+        session->run();
+        WAIT_FOR_STATE(session, DebugSession::EndedState);
+    }
+}
+
 void GdbTest::waitForState(GDBDebugger::DebugSession *session, DebugSession::DebuggerState state,
                             const char *file, int line, bool expectFail)
 {
