@@ -57,7 +57,6 @@
 #include "missingincludemodel.h"
 #include <templateparameterdeclaration.h>
 #include <language/duchain/classdeclaration.h>
-#include "qpropertydeclaration.h"
 #include "model.h"
 #include "helpers.h"
 
@@ -68,8 +67,9 @@
 #include <interfaces/ilanguage.h>
 #include <interfaces/foregroundlock.h>
 
-///Created statically as this object must be a child of the main thread
-CppUtils::ReplaceCurrentAccess accessReplacer;
+using namespace KDevelop;
+
+namespace {
 
 ///If this is enabled, no chain of useless argument-hints for binary operators is created.
 const bool NO_MULTIPLE_BINARY_OPERATORS = true;
@@ -101,6 +101,11 @@ const QSet<QString> BINARY_OPERATORS =
       BINARY_ARITHMETIC_OPERATORS + ARITHMETIC_COMPARISON_OPERATORS;
 //These will be skipped over to find parent contexts
 const QSet<QString> UNARY_OPERATORS = QString("++ -- ! ~ + - & *").split(' ').toSet();
+//When these operators create a parent context it's safe to use their expression type to match against
+//This is only used for the builtin operators, and not the overloaded operators (assuming overloaded operators are found)
+//Add any operators where you expect the lhs type and the rhs type to be the same, at least usually
+//TODO: boolean logic operators could maybe have a fixed match type of boolean
+const QSet<QString> MATCH_TYPE_OPERATORS = QString("!= <= >= = == + - * / % > < -= += *= /= %=").split(' ').toSet();
 const QSet<QString> KEYWORD_ACCESS_STRINGS = QString("const_cast< static_cast< dynamic_cast< reinterpret_cast< const typedef public public: protected protected: private private: virtual return else throw emit Q_EMIT case delete delete[] new friend class namespace").split(' ').toSet();
 //When these appear as access strings, only show types
 const QSet<QString> SHOW_TYPES_ACCESS_STRINGS = QString("const_cast< static_cast< dynamic_cast< reinterpret_cast< const typedef public protected private virtual new friend class").split(' ').toSet();
@@ -121,12 +126,51 @@ const int UNARY_OPERATOR_MATCH = 2; //++
 //Whether identifiers starting with "__" or "_Uppercase" and that are not declared in the current file should be excluded from the code completion
 const bool excludeReservedIdentifiers = true;
 
-using namespace KDevelop;
+/**
+ * A helper "worker" object which lives in the main thread.
+ */
+struct MainThreadHelper : public QObject
+{
+  Q_OBJECT
+
+  public slots:
+    void replaceCurrentAccess(const KUrl& url, const QString& oldAccess, const QString& newAccess);
+};
+
+void MainThreadHelper::replaceCurrentAccess(const KUrl& url, const QString& oldAccess, const QString& newAccess)
+{
+  IDocument* document = ICore::self()->documentController()->documentForUrl(url);
+  if(document) {
+    KTextEditor::Document* textDocument = document->textDocument();
+    if(textDocument) {
+      KTextEditor::View* activeView = textDocument->activeView();
+      if(activeView) {
+        KTextEditor::Cursor cursor = activeView->cursorPosition();
+
+        static KUrl lastUrl;
+        static KTextEditor::Cursor lastPos;
+        if(lastUrl == url && lastPos == cursor) {
+          kDebug() << "Not doing the same access replacement twice at" << lastUrl << lastPos;
+          return;
+        }
+        lastUrl = url;
+        lastPos = cursor;
+
+        KTextEditor::Range oldRange = KTextEditor::Range(cursor - KTextEditor::Cursor(0, oldAccess.length()), cursor);
+        if(oldRange.start().column() >= 0 && textDocument->text(oldRange) == oldAccess) {
+          textDocument->replaceText(oldRange, newAccess);
+        }
+      }
+    }
+  }
+}
+
+static MainThreadHelper s_mainThreadHelper;
+
+}
 
 namespace Cpp {
 
-typedef QPair<Declaration*, int> DeclarationDepthPair;
-
 //Search for a copy-constructor within class
 //*DUChain must be locked*
 bool hasCopyConstructor(CppClassType::Ptr classType, TopDUContext* topContext)
@@ -353,7 +397,7 @@ QString getUnaryOperator(const QString &context)
 }
 
 //Returns the class or struct declaration for the given type
-Declaration* containerDeclForType(AbstractType::Ptr givenType, TopDUContext* top, bool &typeIsPointer)
+Declaration* containerDeclForType(const AbstractType::Ptr& givenType, TopDUContext* top, bool &typeIsPointer)
 {
   if (PointerType::Ptr ptrType = givenType.cast<PointerType>())
   {
@@ -381,38 +425,6 @@ Declaration* containerDeclForType(AbstractType::Ptr givenType, TopDUContext* top
   return 0; //Type could not be identified or was not a class declaration
 }
 
-typedef QPair<Declaration*, bool> DeclAccessPair;
-//Look through localDeclarations of "container" for items matching "type"
-QList<DeclAccessPair> containedItemsMatchingType(Declaration *container, const IndexedType& type, TopDUContext *top, bool isPointer)
-{
-  static const Identifier arrowOpIdentifier("operator->");
-  QList<DeclAccessPair> ret;
-  if (!container || !container->internalContext())
-    return ret;
-  Cpp::TypeConversion conv(top);
-  Declaration *arrowOperator = 0;
-  QVector<Declaration*> containedDeclarations = container->internalContext()->localDeclarations(top);
-  foreach(Declaration *decl, containedDeclarations)
-  {
-    if (decl->isTypeAlias() || decl->isForwardDeclaration() || decl->type<EnumerationType>())
-      continue; //Skip declarations that are not accessed via ./->
-
-    if (!isPointer && decl->identifier() == arrowOpIdentifier)
-      arrowOperator = decl;
-
-    AbstractType::Ptr declEffectiveType = Cpp::effectiveType(decl);
-    if (!declEffectiveType.isNull() && conv.implicitConversion(declEffectiveType->indexed(), type))
-      ret << DeclAccessPair(decl, isPointer);
-  }
-  //If we found an "->", try to treat it as a smart pointer
-  if (arrowOperator)
-  {
-    //containerDeclForType will likely change isPointer... good thing we don't use it again
-    ret += containedItemsMatchingType(containerDeclForType(Cpp::effectiveType(arrowOperator), top, isPointer), type, top, true);
-  }
-  return ret;
-}
-
 CodeCompletionContext::
 CodeCompletionContext( KDevelop::DUContextPointer context, const QString& text,
                        const QString& followingText, const KDevelop::CursorInRevision& position,
@@ -612,6 +624,10 @@ bool CodeCompletionContext::testContextValidity(const QString &expressionPrefix,
     case BinaryOpFunctionCallAccess:
       return m_expressionResult.isInstance;
     case MemberAccess:
+    case ArrowMemberAccess:
+      if (!m_expressionResult.isInstance)
+        return false;
+      //Fall-through
     case MemberChoose:
     case StaticMemberChoose:
       return !m_expression.isEmpty();
@@ -1201,36 +1217,9 @@ QSet<DUContext*> CodeCompletionContext::memberAccessContainers() const {
   return ret;
 }
 
-KDevelop::IndexedType CodeCompletionContext::applyPointerConversionForMatching(KDevelop::IndexedType type, bool fromLValue) const {
-  if(!m_duContext)
-    return KDevelop::IndexedType();
-  
-  if(m_pointerConversionsBeforeMatching == 0)
-    return type;
-  AbstractType::Ptr t = type.abstractType();
-  if(!t)
-    return KDevelop::IndexedType();
-
-  //Can only take addresses of lvalues
-  if(m_pointerConversionsBeforeMatching > 1 || (m_pointerConversionsBeforeMatching && !fromLValue))
-    return IndexedType();
-  
-  if(m_pointerConversionsBeforeMatching > 0) {
-    for(int a = 0; a < m_pointerConversionsBeforeMatching; ++a) {
-      
-      t = TypeUtils::increasePointerDepth(t);
-      if(!t)
-        return IndexedType();
-    }
-  }else{
-    for(int a = m_pointerConversionsBeforeMatching; a < 0; ++a) {
-      t = TypeUtils::decreasePointerDepth(t, m_duContext->topContext());
-      if(!t)
-        return IndexedType();
-    }
-  }
-  
-  return t->indexed();
+int CodeCompletionContext::pointerConversions() const
+{
+  return m_pointerConversionsBeforeMatching;
 }
 
 CodeCompletionContext::~CodeCompletionContext() {
@@ -1365,10 +1354,10 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::memberAccessCompletionIt
       return items;
     ifDebug( kDebug() << "container:" << ctx->scopeIdentifier(true).toString(); )
 
-    foreach( const DeclarationDepthPair& decl,
-            Cpp::hideOverloadedDeclarations(
-              ctx->allDeclarations(ctx->range().end, m_duContext->topContext(), false ),
-              typeIsConst ) )
+    QList<DeclarationDepthPair> decls = ctx->allDeclarations(ctx->range().end, m_duContext->topContext(), false );
+    decls += namespaceItems(ctx, ctx->range().end, false);
+
+    foreach( const DeclarationDepthPair& decl, Cpp::hideOverloadedDeclarations(decls, typeIsConst ) )
     {
       //If we have StaticMemberChoose, which means A::Bla, show only static members, except if we're within a class that derives from the container
       ClassMemberDeclaration* classMember = dynamic_cast<ClassMemberDeclaration*>(decl.first);
@@ -1557,7 +1546,6 @@ QList< CompletionTreeItemPointer > CodeCompletionContext::binaryFunctionAccessCo
       if(decl)
         showName = decl->toString() + " " + m_operator;
     }
-
     items << CompletionTreeItemPointer( new TypeConversionCompletionItem( showName, useType, depth(), KSharedPtr <Cpp::CodeCompletionContext >(this) ) );
   }
 
@@ -1687,8 +1675,15 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::signalSlotAccessCompleti
 
 QList<IndexedType> CodeCompletionContext::matchTypes()
 {
+  if (!m_cachedMatchTypes.isEmpty()) {
+    return m_cachedMatchTypes;
+  }
+
   QSet<KDevelop::IndexedType> ret;
-  if (m_accessType == BinaryOpFunctionCallAccess || m_accessType == FunctionCallAccess )
+  switch(m_accessType)
+  {
+  case BinaryOpFunctionCallAccess:
+  case FunctionCallAccess:
   {
     //MatchTypes for custom operator functions
     foreach(const Function &func, m_matchingFunctionOverloads)
@@ -1699,40 +1694,181 @@ QList<IndexedType> CodeCompletionContext::matchTypes()
       if(funcType && funcType->indexedArgumentsSize() > (uint)func.matchedArguments)
         ret << funcType->indexedArguments()[func.matchedArguments];
     }
-    /* generally matching the other side's type is only useful for "=" or "==" ... Consider:
+    /* generally matching the other side's type is only useful for MATCH_TYPE_OPERATORS ... Consider:
      * if (foo && foo == "str" || <complete here>)
      * The expressionResult preceeding the "||" operator is ["str"] and not [foo == "str"] as it should be
      * therefore the matchType will be "const char *" and not "boolean" as it should be
      * If this is fixed, we could use more operators here. See expressionBefore().
      */
-    if (!m_matchingFunctionOverloads.size() && (m_operator == "=" || m_operator == "=="))
+    if ( !m_matchingFunctionOverloads.size() && MATCH_TYPE_OPERATORS.contains(m_operator) )
       ret << m_expressionResult.type;
+
+    /* A hack: constructor initialization currently is FunctionCallAccess, although obviously the
+     * expression is not a function. Still, we want the match type here, so we check the parents
+     * of this parent and if there we find constructor completion, we can use m_expressionResult
+     */
+    if ( m_expressionResult.isInstance && !m_expressionResult.type.abstractType().cast<FunctionType>() )
+      ret << m_expressionResult.type;
+    break;
   }
-  else if (m_accessType == ReturnAccess)
-  {
+  case ReturnAccess:
     if (AbstractType::Ptr returnType = functionReturnType(m_duContext.data()))
       ret << returnType->indexed();
+    break;
+  case CaseAccess:
+    if (m_expressionResult.isValid() && m_expressionResult.type)
+      ret << m_expressionResult.type;
+  default:
+    break;
   }
-  //As of now, no other access types have any matchTypes
-  return ret.toList();
+
+  m_cachedMatchTypes = ret.toList();
+  return m_cachedMatchTypes;
+}
+
+QList<DeclAccessPair> CodeCompletionContext::containedDeclarationsForLookahead(Declaration* container, TopDUContext* top,
+                                                                               bool isPointer) const
+{
+  static const IndexedIdentifier arrowOpIdentifier(Identifier("operator->"));
+  QList<DeclAccessPair> ret;
+  if (!container || !container->internalContext())
+    return ret;
+
+  Declaration *arrowOperator = 0;
+  QVector<Declaration*> declarations = container->internalContext()->localDeclarations(top);
+  foreach(Declaration *decl, declarations)
+  {
+    if (decl->isTypeAlias() || decl->isForwardDeclaration() || decl->type<EnumerationType>())
+      continue; //Skip declarations that are not accessed via ./->
+
+    if (!isPointer && decl->indexedIdentifier() == arrowOpIdentifier)
+      arrowOperator = decl;
+
+    if (!filterDeclaration(dynamic_cast<ClassMemberDeclaration*>(decl))) {
+      continue;
+    }
+
+    if (Cpp::effectiveType(decl)) {
+      ret << DeclAccessPair(decl, isPointer);
+    }
+  }
+  //If we found an "->", try to treat it as a smart pointer
+  if (arrowOperator) {
+    ret += containedDeclarationsForLookahead( containerDeclForType(Cpp::effectiveType(arrowOperator), top, isPointer),
+                                              top, true );
+  }
+  return ret;
 }
 
 QList<DeclAccessPair> CodeCompletionContext::getLookaheadMatches(Declaration* forDecl, const QList<IndexedType>& matchTypes) const
 {
   QList<DeclAccessPair> ret;
-  // feature disabled for KDevelop 4.5
-  return ret;
-
   if (forDecl->isFunctionDeclaration() || forDecl->kind() != Declaration::Instance || !forDecl->abstractType())
     return ret; //We can only use instances, for now no sub decls of functions either TODO: be nice to get no-arg functions at least
+  TopDUContext* top = m_duContext->topContext();
   bool typeIsPointer = false;
-  Declaration *containerDecl = containerDeclForType(Cpp::effectiveType(forDecl), m_duContext->topContext(), typeIsPointer);
-  foreach(const IndexedType &matchType, matchTypes)
-    ret += containedItemsMatchingType(containerDecl, matchType, m_duContext->topContext(), typeIsPointer);
-  //Could use hideOverloadedDeclarations theoretically here, but it would do very little since we don't have the real declaration depth
+  Declaration* container = containerDeclForType(Cpp::effectiveType(forDecl), top, typeIsPointer);
+  if (!container) {
+    return ret;
+  }
+
+  QHash<Declaration*, QList<DeclAccessPair> >::const_iterator cacheIt = m_lookaheadMatchesCache.constFind(container);
+  if (cacheIt != m_lookaheadMatchesCache.constEnd()) {
+    return cacheIt.value();
+  }
+
+  /// FIXME: use QVector + std::remove_if
+  ret = containedDeclarationsForLookahead(container, top, typeIsPointer);
+
+  QList<DeclAccessPair>::iterator it = ret.begin();
+  Cpp::TypeConversion conv(top);
+  while (it != ret.end()) {
+    bool match = false;
+    const IndexedType& declEffectiveType = Cpp::effectiveType(it->first)->indexed();
+    Q_ASSERT(declEffectiveType);
+
+    foreach (const IndexedType& matchType, matchTypes) {
+      //Don't lookahead if the current type is a (precise) match
+      //Cheaper than checking if it converts, and probably good enough
+      if (matchType == forDecl->indexedType())
+        continue;
+      if (conv.implicitConversion(declEffectiveType, matchType)) {
+        match = true;
+        break;
+      }
+    }
+
+    if (!match) {
+      it = ret.erase(it);
+    } else {
+      ++it;
+    }
+  }
+
+  // Could use hideOverloadedDeclarations theoretically here, but it would do very
+  // little since we don't have the real declaration depth
+
+  m_lookaheadMatchesCache.insert(container, ret);
+
   return ret;
 }
 
+QList<DeclarationDepthPair> CodeCompletionContext::namespaceItems(DUContext* duContext, const CursorInRevision& position, bool global) const
+{
+  QList<DeclarationDepthPair> decls;
+  QList<Declaration*> foundDecls;
+  //Collect the contents of unnamed namespaces
+  if (global) {
+    foundDecls = duContext->findDeclarations(QualifiedIdentifier(unnamedNamespaceIdentifier().identifier()), position);
+  } else {
+    foundDecls = duContext->findLocalDeclarations(unnamedNamespaceIdentifier().identifier(), position);
+  }
+  foreach(Declaration* ns, foundDecls)
+    if(ns->kind() == Declaration::Namespace && ns->internalContext())
+      decls += ns->internalContext()->allDeclarations(position, duContext->topContext(), false);
+
+  //Collect the Declarations from all "using namespace" imported contexts
+  if (global) {
+    foundDecls = duContext->findDeclarations( globalImportIdentifier(), position,
+                                               0, DUContext::NoFiltering );
+  } else {
+    foundDecls = duContext->findLocalDeclarations( globalImportIdentifier(), position,
+                                                   0, AbstractType::Ptr(), DUContext::NoFiltering );
+  }
+
+  QSet<QualifiedIdentifier> ids;
+  foreach(Declaration* importDecl, foundDecls) {
+    NamespaceAliasDeclaration* aliasDecl = dynamic_cast<NamespaceAliasDeclaration*>(importDecl);
+    if(aliasDecl) {
+      ids.insert(aliasDecl->importIdentifier());
+    }else{
+      kDebug() << "Import is not based on NamespaceAliasDeclaration";
+    }
+  }
+
+  QualifiedIdentifier ownNamespaceScope = Cpp::namespaceScopeComponentFromContext(duContext->scopeIdentifier(true), duContext, duContext->topContext());
+  if(!ownNamespaceScope.isEmpty())
+    for(int a = 1; a <= ownNamespaceScope.count(); ++a)
+      ids += ownNamespaceScope.left(a);
+
+  foreach(const QualifiedIdentifier &id, ids) {
+    QList<Declaration*> importedContextDecls = duContext->findDeclarations( id );
+    foreach(Declaration* contextDecl, importedContextDecls) {
+      if(contextDecl->kind() != Declaration::Namespace || !contextDecl->internalContext())
+        continue;
+      DUContext* context = contextDecl->internalContext();
+
+      if(context->range().contains(duContext->range()) && context->url() == duContext->url())
+        continue; //If the context surrounds the current one, the declarations are visible through allDeclarations(..).
+      foreach(Declaration* decl, context->localDeclarations()) {
+        if(filterDeclaration(decl))
+          decls << qMakePair(decl, 1200);
+      }
+    }
+  }
+  return decls;
+}
+
 QList< CompletionTreeItemPointer > CodeCompletionContext::standardAccessCompletionItems() {
   QList<CompletionTreeItemPointer> items;
   LOCKDUCHAIN; if (!m_duContext) return items;
@@ -1745,46 +1881,7 @@ QList< CompletionTreeItemPointer > CodeCompletionContext::standardAccessCompleti
       typeIsConst = true;
   }
   QList<DeclarationDepthPair> decls = m_duContext->allDeclarations(m_duContext->type() == DUContext::Class ? m_duContext->range().end : m_position, m_duContext->topContext());
-
-  //Collect the contents of unnamed namespaces
-  QList<Declaration*> unnamed = m_duContext->findDeclarations(QualifiedIdentifier(unnamedNamespaceIdentifier().identifier()), m_position);
-  foreach(Declaration* ns, unnamed)
-    if(ns->kind() == Declaration::Namespace && ns->internalContext())
-      decls += ns->internalContext()->allDeclarations(m_position, m_duContext->topContext(), false);
-
-  //Collect the Declarations from all "using namespace" imported contexts
-  QList<Declaration*> imports = m_duContext->findDeclarations( globalImportIdentifier(), m_position, 0, DUContext::NoFiltering );
-
-  QSet<QualifiedIdentifier> ids;
-  foreach(Declaration* importDecl, imports) {
-    NamespaceAliasDeclaration* aliasDecl = dynamic_cast<NamespaceAliasDeclaration*>(importDecl);
-    if(aliasDecl) {
-      ids.insert(aliasDecl->importIdentifier());
-    }else{
-      kDebug() << "Import is not based on NamespaceAliasDeclaration";
-    }
-  }
-
-  QualifiedIdentifier ownNamespaceScope = Cpp::namespaceScopeComponentFromContext(m_duContext->scopeIdentifier(true), m_duContext.data(), m_duContext->topContext());
-  if(!ownNamespaceScope.isEmpty())
-    for(int a = 1; a <= ownNamespaceScope.count(); ++a)
-      ids += ownNamespaceScope.left(a);
-
-  foreach(const QualifiedIdentifier &id, ids) {
-    QList<Declaration*> importedContextDecls = m_duContext->findDeclarations( id );
-    foreach(Declaration* contextDecl, importedContextDecls) {
-      if(contextDecl->kind() != Declaration::Namespace || !contextDecl->internalContext())
-        continue;
-      DUContext* context = contextDecl->internalContext();
-
-      if(context->range().contains(m_duContext->range()) && context->url() == m_duContext->url())
-        continue; //If the context surrounds the current one, the declarations are visible through allDeclarations(..).
-      foreach(Declaration* decl, context->localDeclarations()) {
-        if(filterDeclaration(decl))
-          decls << qMakePair(decl, 1200);
-      }
-    }
-  }
+  decls += namespaceItems(m_duContext.data(), m_position, true);
 
   QList<DeclarationDepthPair> oldDecls = decls;
   decls.clear();
@@ -1807,10 +1904,6 @@ QList< CompletionTreeItemPointer > CodeCompletionContext::standardAccessCompleti
     
   decls = Cpp::hideOverloadedDeclarations(decls, typeIsConst);
 
-  QList<IndexedType> matchTypes;
-  if (m_parentContext)
-    matchTypes = parentContext()->matchTypes();
-  QList<CompletionTreeItemPointer> lookaheadMatches;
   foreach( const DeclarationDepthPair& decl, decls ) {
     NormalDeclarationCompletionItem* item = new NormalDeclarationCompletionItem(DeclarationPointer(decl.first), KDevelop::CodeCompletionContext::Ptr(this), decl.second );
 
@@ -1818,42 +1911,6 @@ QList< CompletionTreeItemPointer > CodeCompletionContext::standardAccessCompleti
       item->m_fixedMatchQuality = 0;
 
     items << CompletionTreeItemPointer(item);
-
-    if (!matchTypes.size())
-      continue;
-
-    QList<DeclAccessPair> lookaheadDecls = getLookaheadMatches(decl.first, matchTypes);
-    foreach(const DeclAccessPair &lookaheadDecl, lookaheadDecls)
-    {
-      NormalDeclarationCompletionItem* lookaheadItem =
-          new NormalDeclarationCompletionItem(DeclarationPointer(lookaheadDecl.first), KDevelop::CodeCompletionContext::Ptr(this));
-      lookaheadItem->prefixText = decl.first->identifier().toString() + (lookaheadDecl.second ? "->" : ".");
-      //Perhaps it'd be nice to have these stand out more without polluting the "Best Matches"
-      //NormalDeclarationCompletionItem should be refactored in order to make subclassing simpler
-      lookaheadItem->m_fixedMatchQuality = 0;
-      lookaheadMatches << CompletionTreeItemPointer(lookaheadItem);
-    }
-  }
-  eventuallyAddGroup(i18n("Lookahead Matches"), 100, lookaheadMatches);
-
-  ///Eventually show additional specificly known items for the matched argument-type, like for example enumerators for enum types
-  CodeCompletionContext* parent = parentContext();
-  if(parent) {
-    if(parent->accessType() == FunctionCallAccess) {
-      foreach(const Cpp::OverloadResolutionFunction& function, parent->functions()) {
-        if(function.function.isValid() && function.function.isViable() && function.function.declaration()) {
-          //uint parameterNumber = parent->m_knownArgumentExpressions.size() + function.matchedArguments;
-          Declaration* functionDecl = function.function.declaration().data();
-          if(functionDecl->type<FunctionType>()->arguments().count() > function.matchedArguments) {
-            items += specialItemsForArgumentType(functionDecl->type<FunctionType>()->arguments()[function.matchedArguments]);
-          }
-        }
-      }
-    }
-    else if (parent->accessType() == BinaryOpFunctionCallAccess || parent->accessType() == CaseAccess)
-    {
-      items += specialItemsForArgumentType(parent->m_expressionResult.type.abstractType());
-    }
   }
 
   return items;
@@ -1895,7 +1952,7 @@ void CodeCompletionContext::addCPPBuiltin()
   }
 
   ClassFunctionDeclaration* classFun = dynamic_cast<ClassFunctionDeclaration*>(DUChainUtils::declarationForDefinition(functionContext->owner(), m_duContext->topContext()));
-  
+
   if(classFun && !classFun->isStatic() && classFun->context()->owner()
               && m_onlyShow != ShowSignals && m_onlyShow != ShowSlots && m_onlyShow != ShowTypes)
   {
@@ -1984,9 +2041,21 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& sh
         break;
     }
 
+    LOCKDUCHAIN; if (!m_duContext) return items;
+    if (m_accessType == MemberAccess ||
+        m_accessType == ArrowMemberAccess ||
+        m_accessType == MemberChoose ||
+        m_accessType == NoMemberAccess)
+      addLookaheadMatches(items);
+
+    if (parentContext()) {
+      foreach(const IndexedType &matchType, parentContext()->matchTypes()) {
+        addSpecialItemsForArgumentType(matchType.abstractType());
+      }
+    }
+
     if(depth() == 0)
     {
-      LOCKDUCHAIN; if (!m_duContext) return items;
       //Eventually add missing include-completion in cases like SomeNamespace::NotIncludedClass|
       if(m_accessType == StaticMemberChoose) {
 #ifndef TEST_COMPLETION
@@ -2003,6 +2072,38 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& sh
     return items;
 }
 
+void CodeCompletionContext::addLookaheadMatches(const QList<CompletionTreeItemPointer> items)
+{
+  QList<IndexedType> matchTypes;
+  if (m_parentContext)
+    matchTypes = parentContext()->matchTypes();
+
+  if (!matchTypes.size())
+    return;
+
+  QList<CompletionTreeItemPointer> lookaheadMatches;
+  foreach( const CompletionTreeItemPointer &item, items ) {
+    Declaration* decl = item->declaration().data();
+    if (!decl)
+      continue;
+
+    QList<DeclAccessPair> lookaheadDecls = getLookaheadMatches(decl, matchTypes);
+    foreach(const DeclAccessPair &lookaheadDecl, lookaheadDecls)
+    {
+      NormalDeclarationCompletionItem* lookaheadItem =
+          new NormalDeclarationCompletionItem(DeclarationPointer(lookaheadDecl.first), KDevelop::CodeCompletionContext::Ptr(this));
+      lookaheadItem->prefixText = decl->identifier().toString() + (lookaheadDecl.second ? "->" : ".");
+      //Perhaps it'd be nice to have these stand out more without polluting the "Best Matches"
+      //NormalDeclarationCompletionItem should be refactored in order to make subclassing simpler
+      lookaheadItem->m_fixedMatchQuality = 0;
+      lookaheadMatches << CompletionTreeItemPointer(lookaheadItem);
+    }
+  }
+  m_lookaheadMatchesCache.clear();
+
+  eventuallyAddGroup(i18n("Lookahead Matches"), 800, lookaheadMatches);
+}
+
 QList<CompletionTreeItemPointer> CodeCompletionContext::getImplementationHelpers() {
   QList<CompletionTreeItemPointer> ret;
   TopDUContext* searchInContext = m_duContext->topContext();
@@ -2011,7 +2112,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::getImplementationHelpers
     ret += getImplementationHelpersInternal(m_duContext->scopeIdentifier(true), searchInContext);
 
   if(!CppUtils::isHeader( searchInContext->url().toUrl() )) {
-    KUrl headerUrl = CppUtils::sourceOrHeaderCandidate( searchInContext->url().toUrl(), true );
+    KUrl headerUrl = CppUtils::sourceOrHeaderCandidate( searchInContext->url().toUrl(), false );
     searchInContext = ICore::self()->languageController()->language("C++")->languageSupport()->standardContext(headerUrl);
     if(searchInContext)
       ret += getImplementationHelpersInternal(m_duContext->scopeIdentifier(true), searchInContext);
@@ -2057,27 +2158,25 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::getImplementationHelpers
   return ret;
 }
 
-QList< KSharedPtr< KDevelop::CompletionTreeItem > > CodeCompletionContext::specialItemsForArgumentType(TypePtr< KDevelop::AbstractType > type) {
-  QList< KSharedPtr< KDevelop::CompletionTreeItem > > items;
+void CodeCompletionContext::addSpecialItemsForArgumentType(AbstractType::Ptr type) {
+  QList< CompletionTreeItemPointer > items;
   if(EnumerationType::Ptr enumeration = TypeUtils::realType(type, m_duContext->topContext()).cast<EnumerationType>()) {
     Declaration* enumDecl = enumeration->declaration(m_duContext->topContext());
     if(enumDecl && enumDecl->internalContext()) {
-
-      // This is necessary to avoid duplication of locally visible enumerators, which also get added by standardAccessCompletionItems().
-      if( visibleFromWithin( enumDecl, m_duContext.data() ) )
-        return items;
-
       DUContext* enumInternal = enumDecl->internalContext();
       foreach(Declaration* enumerator, enumInternal->localDeclarations()) {
-        items << CompletionTreeItemPointer(new NormalDeclarationCompletionItem( DeclarationPointer(enumerator), KDevelop::CodeCompletionContext::Ptr(this), 0 ));
-        static_cast<NormalDeclarationCompletionItem*>(items.back().data())->prependScopePrefix = true;
+        NormalDeclarationCompletionItem *item =
+            new NormalDeclarationCompletionItem( DeclarationPointer(enumerator), KDevelop::CodeCompletionContext::Ptr(this), 0 );
+        item->prependScopePrefix = true;
+        item->m_fixedMatchQuality = 0;
+        items << CompletionTreeItemPointer(item);
       }
     }
   }
-  return items;
+  eventuallyAddGroup("Enum values", 0, items);
 }
 
-bool CodeCompletionContext::visibleFromWithin(KDevelop::Declaration* decl, DUContext* currentContext) {
+bool CodeCompletionContext::visibleFromWithin(Declaration* decl, DUContext* currentContext) const {
   if(!decl || !currentContext)
     return false;
   if(currentContext->imports(decl->context()))
@@ -2086,7 +2185,7 @@ bool CodeCompletionContext::visibleFromWithin(KDevelop::Declaration* decl, DUCon
   return visibleFromWithin(decl, currentContext->parentContext());
 }
 
-bool CodeCompletionContext::isIntegralConstant(Declaration* decl, bool acceptHelperItems) {
+bool CodeCompletionContext::isIntegralConstant(Declaration* decl, bool acceptHelperItems) const {
 
     // Usability issue: if we're matching for integral constants,
     // types and functions are also allowed (see filterDeclaration()),
@@ -2148,7 +2247,7 @@ static inline int getIntegralReturnType(const AbstractType::Ptr& type)
   return intType->dataType();
 }
 
-bool  CodeCompletionContext::filterDeclaration(Declaration* decl, DUContext* declarationContext, bool dynamic) {
+bool  CodeCompletionContext::filterDeclaration(Declaration* decl, DUContext* declarationContext, bool dynamic) const {
   if(!decl)
     return true;
 
@@ -2222,7 +2321,7 @@ bool  CodeCompletionContext::filterDeclaration(Declaration* decl, DUContext* dec
       && getIntegralReturnType(decl->abstractType()) == IntegralType::TypeVoid)
   {
     const ExpressionEvaluationResult& result =
-            static_cast<CodeCompletionContext*>(m_parentContext.data())->m_expressionResult;
+            static_cast<const CodeCompletionContext*>(m_parentContext.data())->m_expressionResult;
     // for now only hide in non-lvalue expressions so we don't get problems in sig/slot connections e.g.
     if (result.type.isValid() && !result.isLValue())
       return false;
@@ -2231,22 +2330,19 @@ bool  CodeCompletionContext::filterDeclaration(Declaration* decl, DUContext* dec
   return true;
 }
 
-bool  CodeCompletionContext::filterDeclaration(ClassMemberDeclaration* decl, DUContext* declarationContext) {
+bool CodeCompletionContext::filterDeclaration(ClassMemberDeclaration* decl, DUContext* declarationContext) const {
   if(m_doAccessFiltering && decl) {
     if(!Cpp::isAccessible(m_localClass ? m_localClass.data() : m_duContext.data(), decl, m_duContext->topContext(), declarationContext))
       return false;
   }
-  // filter properties from code completion, they mostly have to be accessed via their getter/setters
-  if (dynamic_cast<QPropertyDeclaration*>(decl)) {
-    return false;
-  }
   return filterDeclaration((Declaration*)decl, declarationContext, false);
 }
 
-void CodeCompletionContext::replaceCurrentAccess(QString old, QString _new)
+void CodeCompletionContext::replaceCurrentAccess(const QString& old, const QString& _new)
 {
   //We must not change the document from within the background, so we use a queued connection to an object created in the foregroud
-  QMetaObject::invokeMethod(&accessReplacer, "exec", Qt::QueuedConnection, Q_ARG(KUrl, m_duContext->url().toUrl()), Q_ARG(QString, old), Q_ARG(QString, _new));
+  QMetaObject::invokeMethod(&s_mainThreadHelper, "replaceCurrentAccess", Qt::QueuedConnection,
+                            Q_ARG(KUrl, m_duContext->url().toUrl()), Q_ARG(QString, old), Q_ARG(QString, _new));
 }
 
 int CodeCompletionContext::matchPosition() const {
@@ -2395,3 +2491,5 @@ void CodeCompletionContext::setFollowingText(QString str) {
 
 
 }
+
+#include "context.moc"
