@@ -34,8 +34,6 @@ using namespace KDevelop;
 #define ENSURE_READ_LOCKED   ENSURE_FILE_READ_LOCKED(*this)
 #define ENSURE_WRITE_LOCKED   if(indexedTopContext().isValid()) { ENSURE_CHAIN_READ_LOCKED }
 
-// #define LEXERCACHE_DEBUG
-
 DEFINE_LIST_MEMBER_HASH(IncludePathListItem, m_includePaths, KDevelop::IndexedString)
 
 struct IncludePathListItem {
@@ -89,12 +87,12 @@ struct IncludePathListItem {
 typedef AppendedListItemRequest<IncludePathListItem, 40*4> IncludePathsRequest;
 
 typedef KDevelop::ItemRepository<IncludePathListItem, IncludePathsRequest> IncludePathsRepository;
-IncludePathsRepository includePathsRepository("include path repository");
+IncludePathsRepository& includePathsRepository()
+{
+  static IncludePathsRepository repo("include path repository");
+  return repo;
+}
 
-bool Cpp::EnvironmentManager::m_simplifiedMatching = false;
-Cpp::EnvironmentManager::MatchingLevel Cpp::EnvironmentManager::m_matchingLevel = Cpp::EnvironmentManager::Full;
-//  #define LEXERCACHE_DEBUG
-//   #define ifDebug(X) X
 //If DYNAMIC_DEBUGGING is defined, debugging can be started at any point in runtime,
 //by calling setIsDebugging(true) from within the debugger
 // #define DYNAMIC_DEBUGGING
@@ -111,7 +109,7 @@ void setIsDebugging(bool is) {
   is_debugging = is;
 }
 
-#define LEXERCACHE_DEBUG
+#define DEBUG_LEXERCACHE
 #define ifDebug(x) if(debugging()) {x;}
 #else
 inline bool debugging() {
@@ -123,31 +121,24 @@ using namespace Cpp;
 
 REGISTER_DUCHAIN_ITEM(EnvironmentFile);
 
-//Repository that contains the actual macros, and maps them to indices
-MacroDataRepository Cpp::EnvironmentManager::macroDataRepository("macro repository");
-//Set-repository that contains the string-sets
-Utils::StringSetRepository Cpp::EnvironmentManager::stringSetRepository("string sets");
-//Set-repository that contains the macro-sets
-MacroSetRepository Cpp::EnvironmentManager::macroSetRepository;
-
 namespace Cpp {
   Utils::BasicSetRepository* StaticStringSetRepository::repository() {
-    return &Cpp::EnvironmentManager::stringSetRepository;
+    return &Cpp::EnvironmentManager::self()->stringSetRepository();
   }
   Utils::BasicSetRepository* StaticMacroSetRepository::repository() {
-    return &Cpp::EnvironmentManager::macroSetRepository;
+    return &Cpp::EnvironmentManager::self()->macroSetRepository();
   }
   void MacroSetRepository::itemRemovedFromSets(uint index) {
-    Cpp::EnvironmentManager::macroDataRepository.deleteItem(index);
+    Cpp::EnvironmentManager::self()->macroDataRepository().deleteItem(index);
   }
 }
 
 const rpp::pp_macro& Cpp::MacroIndexConversion::toItem(uint index) const {
-  return *EnvironmentManager::macroDataRepository.itemFromIndex( index );
+  return *EnvironmentManager::self()->macroDataRepository().itemFromIndex( index );
 }
 
 uint Cpp::MacroIndexConversion::toIndex(const rpp::pp_macro& macro) const {
-  return EnvironmentManager::macroDataRepository.index( MacroRepositoryItemRequest(macro) );
+  return EnvironmentManager::self()->macroDataRepository().index( MacroRepositoryItemRequest(macro) );
 }
 
 //For debugging
@@ -185,22 +176,31 @@ QString print(const Cpp::ReferenceCountedMacroSet& set) {
   return ret;
 }
 
-void EnvironmentManager::setSimplifiedMatching(bool simplified) {
-  m_simplifiedMatching = simplified;
+EnvironmentManager* EnvironmentManager::m_self = 0;
+
+EnvironmentManager::EnvironmentManager()
+  : m_matchingLevel(Full), m_simplifiedMatching(false),
+    m_macroDataRepository("macro repository"), m_stringSetRepository("string sets"), m_macroSetRepository()
+{
 }
 
-Cpp::EnvironmentManager::MatchingLevel Cpp::EnvironmentManager::matchingLevel() {
-  return m_matchingLevel;
+void EnvironmentManager::init()
+{
+  Q_ASSERT_X(!m_self, "EnvironmentManager::init()", "do not call init() twice");
+  m_self = new EnvironmentManager();
+  // also init the other getters
+  includePathsRepository();
+}
+
+void EnvironmentManager::setSimplifiedMatching(bool simplified)
+{
+  m_simplifiedMatching = simplified;
 }
 
 void Cpp::EnvironmentManager::setMatchingLevel(Cpp::EnvironmentManager::MatchingLevel level) {
   m_matchingLevel = level;
 }
 
-bool EnvironmentManager::isSimplifiedMatching() {
-  return m_simplifiedMatching;
-}
-
 bool EnvironmentFile::matchEnvironment(const ParsingEnvironment* _environment) const {
   ENSURE_READ_LOCKED
   const CppPreprocessEnvironment* cppEnvironment = dynamic_cast<const CppPreprocessEnvironment*>(_environment);
@@ -208,20 +208,20 @@ bool EnvironmentFile::matchEnvironment(const ParsingEnvironment* _environment) c
     return false;
 
   if( cppEnvironment->identityOffsetRestrictionEnabled() && cppEnvironment->identityOffsetRestriction() != identityOffset() ) {
-#ifdef LEXERCACHE_DEBUG
+#ifdef DEBUG_LEXERCACHE
     kDebug( 9007 ) << "file" << url().str() << "does not match branching hash. Restriction:" << cppEnvironment->identityOffsetRestriction() << "Actual:" << identityOffset();
 #endif
     return false;
   }
   
-  if(EnvironmentManager::matchingLevel() == EnvironmentManager::Disabled)
+  if(EnvironmentManager::self()->matchingLevel() == EnvironmentManager::Disabled)
     return true;
   
   //Consider files that are out-guarded by the header-guard as a match, without looking into their content
   ///@todo Pick the version that is already in the environment if there is multiple
-  if(EnvironmentManager::matchingLevel() == EnvironmentManager::Naive)
+  if(EnvironmentManager::self()->matchingLevel() == EnvironmentManager::Naive)
     if(cppEnvironment->macroNameSet().contains(headerGuard())) {
-#ifdef LEXERCACHE_DEBUG
+#ifdef DEBUG_LEXERCACHE
       kDebug( 9007 ) << "file" << url().str() << "environment contains the header-guard, returning true";
 #endif
       return true;
@@ -235,7 +235,7 @@ bool EnvironmentFile::matchEnvironment(const ParsingEnvironment* _environment) c
     rpp::pp_macro* m = cppEnvironment->retrieveStoredMacro( *it );
     if(m && !m->isUndef()) {
       
-#ifdef LEXERCACHE_DEBUG
+#ifdef DEBUG_LEXERCACHE
       if(debugging()) {
         kDebug(9007) << "The environment contains a macro that can affect the cached file, but that should not exist:" << m->name.str();
       }
@@ -289,13 +289,13 @@ bool EnvironmentFile::needsUpdate(const ParsingEnvironment* environment) const {
   
   //When in naive matching mode, we even use the non-guarded version when inappropriate. We must make sure not to update it in such
   //a situation, else it will end up empty
-  if(cppEnvironment && EnvironmentManager::matchingLevel() <= EnvironmentManager::Naive && !headerGuard().isEmpty() && cppEnvironment->macroNameSet().contains(headerGuard()))
+  if(cppEnvironment && EnvironmentManager::self()->matchingLevel() <= EnvironmentManager::Naive && !headerGuard().isEmpty() && cppEnvironment->macroNameSet().contains(headerGuard()))
     return false;
   
   return ParsingEnvironmentFile::needsUpdate(environment) || d_func()->m_includePathDependencies.needsUpdate();
 }
 
-EnvironmentFile::EnvironmentFile( IndexedString url, TopDUContext* topContext ) : ParsingEnvironmentFile(*new EnvironmentFileData(), url) {
+EnvironmentFile::EnvironmentFile( const IndexedString& url, TopDUContext* topContext ) : ParsingEnvironmentFile(*new EnvironmentFileData(), url) {
 
   d_func_dynamic()->setClassId(this);
   setLanguage(IndexedString("C++"));
@@ -328,7 +328,7 @@ int EnvironmentFile::contentStartLine() const {
 
 void EnvironmentFile::addDefinedMacro( const rpp::pp_macro& macro, const rpp::pp_macro* previousOfSameName ) {
   ENSURE_WRITE_LOCKED
-#ifdef LEXERCACHE_DEBUG
+#ifdef DEBUG_LEXERCACHE
   if(debugging()) {
   kDebug( 9007 )  << id(this) << "defined macro" << macro.name.str();
   }
@@ -362,7 +362,7 @@ void EnvironmentFile::addDefinedMacro( const rpp::pp_macro& macro, const rpp::pp
 void EnvironmentFile::usingMacro( const rpp::pp_macro& macro ) {
   ENSURE_WRITE_LOCKED
   if ( !d_func()->m_definedMacroNames.contains( macro.name ) && !d_func()->m_unDefinedMacroNames.contains( macro.name ) && !macro.isUndef() ) {
-#ifdef LEXERCACHE_DEBUG
+#ifdef DEBUG_LEXERCACHE
   if(debugging()) {
     kDebug( 9007 ) << id(this) << "used macro" << macro.name.str() << "from" << macro.file.str();
   }
@@ -416,7 +416,7 @@ const QList<IndexedString> EnvironmentFile::includePaths() const {
   ENSURE_READ_LOCKED
   QList<IndexedString> ret;
   if(d_func()->m_includePaths) {
-    const IncludePathListItem* item = includePathsRepository.itemFromIndex(d_func()->m_includePaths);
+    const IncludePathListItem* item = includePathsRepository().itemFromIndex(d_func()->m_includePaths);
     
     FOREACH_FUNCTION(const IndexedString& include, item->m_includePaths)
       ret << include;
@@ -427,21 +427,21 @@ const QList<IndexedString> EnvironmentFile::includePaths() const {
 void EnvironmentFile::setIncludePaths( const QList<IndexedString>& paths ) {
   ENSURE_WRITE_LOCKED
   
-  QMutexLocker lock(includePathsRepository.mutex());
+  QMutexLocker lock(includePathsRepository().mutex());
   
   if(d_func()->m_includePaths) {
-    KDevelop::DynamicItem<IncludePathListItem, true> item = includePathsRepository.dynamicItemFromIndex(d_func()->m_includePaths);
+    KDevelop::DynamicItem<IncludePathListItem, true> item = includePathsRepository().dynamicItemFromIndex(d_func()->m_includePaths);
     --item->m_refCount;
     if(!item->m_refCount)
-      includePathsRepository.deleteItem(d_func()->m_includePaths);
+      includePathsRepository().deleteItem(d_func()->m_includePaths);
     d_func_dynamic()->m_includePaths = 0;
   }
   if(!paths.isEmpty()) {
     IncludePathListItem item;
     foreach(const IndexedString &include, paths)
       item.m_includePathsList().append(include);
-    d_func_dynamic()->m_includePaths = includePathsRepository.index(item);
-    KDevelop::DynamicItem<IncludePathListItem, true> gotItem = includePathsRepository.dynamicItemFromIndex(d_func()->m_includePaths);
+    d_func_dynamic()->m_includePaths = includePathsRepository().index(item);
+    KDevelop::DynamicItem<IncludePathListItem, true> gotItem = includePathsRepository().dynamicItemFromIndex(d_func()->m_includePaths);
     ++gotItem->m_refCount;
   }
 }
@@ -487,12 +487,12 @@ void EnvironmentFile::addStrings( const std::set<Utils::BasicSetRepository::Inde
 }
 
 //The parameter should be a EnvironmentFile that was lexed AFTER the content of this file
-void EnvironmentFile::merge( const EnvironmentFile& file, CppPreprocessEnvironment* env ) {
+void EnvironmentFile::merge( const EnvironmentFile& file ) {
   ENSURE_WRITE_LOCKED
   //We have to read the other file
   ENSURE_FILE_READ_LOCKED(file)
   
-#ifdef LEXERCACHE_DEBUG
+#ifdef DEBUG_LEXERCACHE
   if(debugging()) {
   kDebug( 9007 ) <<  id(this) << ": merging" << id(&file)  << "defined in macros this:" << print(d_func()->m_definedMacroNames)  << "defined macros in other:" << print(file.d_func()->m_definedMacroNames) << "undefined macros in other:" << print(file.d_func()->m_unDefinedMacroNames) << "strings in other:" << print(file.strings());
   }
@@ -584,7 +584,7 @@ void EnvironmentFile::merge( const EnvironmentFile& file, CppPreprocessEnvironme
   
   addModificationRevisions(file.allModificationRevisions());
 
-#ifdef LEXERCACHE_DEBUG
+#ifdef DEBUG_LEXERCACHE
   if(debugging()) {
   kDebug( 9007 ) << id(this) << ": defined macro names in this after merge:" << d_func()->m_definedMacroNames.set().count() << print(d_func()->m_definedMacroNames);
   kDebug( 9007 ) << id(this) << ": defined in this after merge:" << d_func()->m_definedMacros.set().count() << print(d_func()->m_definedMacros);
