@@ -78,8 +78,11 @@ namespace Cpp {
 AbstractType::Ptr applyPointerReference( AbstractType::Ptr ptr, const KDevelop::IndexedTypeIdentifier& id ) {
   AbstractType::Ptr ret = ptr;
 
-  if(ret && static_cast<bool>(ret->modifiers() & AbstractType::ConstModifier) != id.isConstant())
-    ret->setModifiers(id.isConstant() ? AbstractType::ConstModifier : AbstractType::NoModifiers);
+  if(ret && ((static_cast<bool>(ret->modifiers() & AbstractType::ConstModifier) != id.isConstant())
+         || (static_cast<bool>(ret->modifiers() & AbstractType::VolatileModifier) != id.isVolatile()))) {
+    ret->setModifiers((id.isConstant() ? AbstractType::ConstModifier : AbstractType::NoModifiers)
+                    | (id.isVolatile() ? AbstractType::VolatileModifier : AbstractType::NoModifiers));
+  }
 
   for( int a = 0; a < id.pointerDepth(); ++a ) {
     uint modifiers = AbstractType::NoModifiers;
@@ -95,7 +98,9 @@ AbstractType::Ptr applyPointerReference( AbstractType::Ptr ptr, const KDevelop::
   if(id.isReference() ) {
     uint modifiers = AbstractType::NoModifiers;
     if( id.isConstant() )
-      modifiers = AbstractType::ConstModifier;
+      modifiers |= AbstractType::ConstModifier;
+    if( id.isVolatile() )
+      modifiers |= AbstractType::VolatileModifier;
 
     ReferenceType::Ptr newRet( new ReferenceType() );
     newRet->setModifiers(modifiers);
@@ -160,7 +165,6 @@ struct ThreadLocalData {
   uint delayedDepth;
   // recursion counter for alias type resolution
   uint aliasDepth;
-  uint specializedFromDepth;
 };
 
 #if (QT_VERSION >= 0x040801)
@@ -258,7 +262,18 @@ struct DelayedTypeResolver : public KDevelop::TypeExchanger
       else
         res = p.evaluateType( delayedType->identifier().toString().toUtf8(), DUContextPointer(const_cast<DUContext*>(searchContext)), source );
 
-      return res.type.abstractType();
+      // NOTE: This looks hacky, but see e.g. TestDUChain::testDecltypeTypedef - we really _never_
+      //       can replace a delayed type with a CppTemplateParameterType. Instead we need to create
+      //       a new delayed type for that with its identifier.
+      //TODO: try to add support for replacing CppTemplateParameterTypes directly eventually...
+      //      dunno if its actually doable though
+      AbstractType::Ptr ret = res.type.abstractType();
+      if (CppTemplateParameterType::Ptr tplParam = ret.cast<CppTemplateParameterType>()) {
+        DelayedType::Ptr delayed(new DelayedType);
+        delayed->setIdentifier(IndexedTypeIdentifier(tplParam->declarationId().qualifiedIdentifier().last().toString()));
+        return delayed.cast<AbstractType>();
+      }
+      return ret;
     }else{
       if( containsDelayedType(type) )
       {
@@ -375,30 +390,7 @@ TemplateDeclaration* TemplateDeclaration::instantiatedFrom() const {
 }
 
 void TemplateDeclaration::setSpecializedFrom(TemplateDeclaration* other) {
-  ThreadLocalData& data = threadDataLocal();
-  PushValue<uint> safety(data.specializedFromDepth, data.specializedFromDepth+1);
-  if(data.specializedFromDepth > 30) {
-    QString withVal = "other";
-    if( other ) {
-      if( other->instantiatedFrom() ) {
-        withVal += ".instantiatedFrom() == " + other->instantiatedFrom()->id(true).qualifiedIdentifier().toString();
-      } else if( other->specializedFrom().data() ) {
-        withVal += ".specialiedFrom().data() == " + other->specializedFrom().data()->identifier().toString();
-      } else {
-        withVal += ".specializedFrom().data() == null && other.instantiatedFrom() == null";
-      }
-    } else {
-      withVal = " == null";
-    }
-    kWarning()
-        << "depth-limit reached while setting specializedFrom"
-        << (specializedFrom().data()
-            ? specializedFrom().data()->identifier().toString()
-            : "this specializedFrom is null")
-        << "with" << withVal;
-    return;
-  }
-
+  
   if(other && other->instantiatedFrom()) {
     setSpecializedFrom(other->instantiatedFrom());
     return;
@@ -616,6 +608,7 @@ CppDUContext<KDevelop::DUContext>* instantiateDeclarationAndContext( KDevelop::D
             ///Resolve the delayed type, and import the context
             DelayedTypeResolver res(contextCopy, source);
             AbstractType::Ptr newType( res.exchange(delayed.cast<AbstractType>()) );
+            newType = TypeUtils::unAliasedType(newType);
 
             if( CppClassType::Ptr baseClass = newType.cast<CppClassType>() )
             {
@@ -627,7 +620,7 @@ CppDUContext<KDevelop::DUContext>* instantiateDeclarationAndContext( KDevelop::D
               newInstance.baseClass = newType->indexed();
               klass->replaceBaseClass( num, newInstance );
             } else {
-//               kDebug(9007) << "Resolved bad base-class";
+              kWarning(9007) << "Resolved bad base-class" << delayed->toString() << (newType ? newType->toString() : QString());
             }
           }
           ++num;
@@ -767,7 +760,6 @@ void TemplateDeclaration::deleteAllInstantiations()
   foreach( TemplateDeclaration* decl, instantiations ) {
     Q_ASSERT(decl);
     decl->m_instantiatedFrom = 0;
-    Declaration* realDecl = dynamic_cast<Declaration*>(decl);
     //Only delete real insantiations, not specializations
     //FIXME: before this checked for decl->isAnonymous
     //This was a problem because some instantiations are not anonymous, so they end up orphaned from their m_instantiatedFrom
