@@ -83,6 +83,8 @@ private slots:
   void testSearchAcrossNamespace2();
   void testSearchAcrossNamespace3();
   void testADL();
+  void testADLConstness();
+  void testADLConstness2();
   void testADLClassType();
   void testADLFunctionType();
   void testADLFunctionByName();
@@ -94,6 +96,8 @@ private slots:
   void testADLNameAlias();
   void testADLTemplateArguments();
   void testADLTemplateTemplateArguments();
+  void testADLEllipsis();
+  void testAssignmentOperators();
   void testTemplateEnums();
   void testIntegralTemplates();
   void testTypedef();
@@ -109,6 +113,8 @@ private slots:
   void testTemplates2();
   void testTemplates3();
   void testTemplates4();
+  void testSpecializationSelection();
+  void testSpecializationSelection2();
   void testTemplatesRebind();
   void testTemplatesRebind2();
   void testTemplateDefaultParameters();
@@ -116,12 +122,14 @@ private slots:
   void testFunctionTemplates();
   void testContextAssignment();
   void testSpecializedTemplates();
+  void testTemplatesSuper();
   void testFunctionDefinition();
   void testFunctionDefinition2();
   void testFunctionDefinition3();
   void testFunctionDefinition4();
   void testFunctionDefinition5();
   void testFunctionDefinition6();
+  void testFunctionDefinition7();
   void testMetaProgramming();
   void testMetaProgramming2();
   void testMetaProgramming3();
@@ -173,6 +181,7 @@ private slots:
   void testContextSearch();
 
   void testEllipsis();
+  void testEllipsisVexing();
 
   void testMultiByteCStrings();
 
@@ -190,16 +199,21 @@ private slots:
   
   void testDeclarationHasUses();
   void testBug269352();
+  void testRenameClass();
+
+  void testQProperty();
 
   //BEGIN C++2011
   void testRangeBasedFor();
   void testRangeBasedForClass();
+  void testRangeBasedForClass2();
   void testRValueReference();
   void testDefaultDelete();
   void testDelete_Bug278781();
   void testEnum2011_data();
   void testEnum2011();
   void testDecltype();
+  void testDecltypeTypedef();
   void testDecltypeUses();
   void testTrailingReturnType();
   void testConstexpr();
@@ -211,9 +225,11 @@ private slots:
   void testLambdaCapture();
   void testTemplateSpecializeArray();
   void testTemplateSpecializeRValue();
+  void testTemplateSpecializeVolatile();
   void testAliasDeclaration();
   void testAuto();
   void testNoexcept();
+  void testInlineNamespace();
   //END C++2011
 
 private:
