@@ -35,10 +35,13 @@
 
 #include "typeutils.h"
 #include "templatedeclaration.h"
+#include "cppduchain.h"
 
 using namespace KDevelop;
 using namespace Cpp;
 
+extern template char *QTest::toString(const KDevelop::RangeInRevision &range);
+
 void TestDUChain::testRangeBasedFor() {
   //                                         1         2         3
   //                                123456789012345678901234567890
@@ -124,16 +127,78 @@ void TestDUChain::testRangeBasedForClass() {
   decls = ctx->childContexts().at(0)->localDeclarations();
   QCOMPARE(decls.size(), 1);
   QCOMPARE(decls.at(0)->identifier().toString(), QString("i"));
+  QVERIFY(decls.at(0)->abstractType());
   QCOMPARE(decls.at(0)->abstractType()->toString(), QString("MyType"));
   QCOMPARE(decls.at(0)->range().start.line, 10);
 
   decls = ctx->childContexts().at(2)->localDeclarations();
   QCOMPARE(decls.size(), 1);
   QCOMPARE(decls.at(0)->identifier().toString(), QString("i"));
+  QVERIFY(decls.at(0)->abstractType());
   QCOMPARE(decls.at(0)->abstractType()->toString(), QString("MyType"));
   QCOMPARE(decls.at(0)->range().start.line, 12);
 }
 
+void TestDUChain::testRangeBasedForClass2() {
+  // see: https://bugs.kde.org/show_bug.cgi?id=299310
+  // see: 24.6.5 in the c++11 standard: range access [iterator.range]
+  //                                         1         2         3
+  //                                123456789012345678901234567890
+  LockedTopDUContext top = parse(  "template<typename T> struct list{"
+                                 "\n  T*& begin();"
+                                 "\n  const T* const& begin() const;"
+                                 "\n};"
+                                 "\nnamespace std{"
+                                 "\n  template<class C> auto begin(C& c) -> decltype(c.begin());"
+                                 "\n  template<class C> auto begin(const C& c) -> decltype(c.begin());"
+                                 "\n}"
+                                 "\nint main() {"
+                                 "\n  list<double> l1;"
+                                 "\n  int a = 0;" // make sure lastType is bogus
+                                 "\n  for(auto i : l1) { }"
+                                 "\n  const list<float> l2;"
+                                 "\n  int b = 0;" // make sure lastType is bogus
+                                 "\n  for(auto i : l2) { }"
+                                 "\n}"
+                                 "\n", DumpNone
+                           );
+  QVERIFY(top);
+  DUChainReadLocker lock;
+  QVERIFY(top->problems().isEmpty());
+  QCOMPARE(top->childContexts().size(), 5);
+  QCOMPARE(top->localDeclarations().size(), 3);
+  DUContext* ctx = top->childContexts().last();
+  QVector< Declaration* > decls = ctx->localDeclarations();
+  QCOMPARE(decls.size(), 4);
+
+  QCOMPARE(decls.at(0)->identifier().toString(), QString("l1"));
+  QCOMPARE(decls.at(0)->abstractType()->toString(), QString("list< double >"));
+  QCOMPARE(decls.at(0)->uses().constBegin()->size(), 1);
+  QCOMPARE(decls.at(0)->uses().constBegin()->first().start.line, 11);
+
+  QCOMPARE(decls.at(2)->identifier().toString(), QString("l2"));
+  QCOMPARE(decls.at(2)->abstractType()->toString(), QString("const list< float >"));
+  QCOMPARE(decls.at(2)->uses().constBegin()->size(), 1);
+  QCOMPARE(decls.at(2)->uses().constBegin()->first().start.line, 14);
+
+  QCOMPARE(ctx->childContexts().size(), 4);
+  decls = ctx->childContexts().at(0)->localDeclarations();
+  QCOMPARE(decls.size(), 1);
+  QCOMPARE(decls.at(0)->identifier().toString(), QString("i"));
+  QVERIFY(decls.at(0)->abstractType());
+  QCOMPARE(decls.at(0)->abstractType()->toString(), QString("double"));
+  QCOMPARE(decls.at(0)->range().start.line, 11);
+
+  // note: auto e = begin(const list<>&) will return a *non* const type
+  // this is expected as per definition of the auto functionality
+  decls = ctx->childContexts().at(2)->localDeclarations();
+  QCOMPARE(decls.size(), 1);
+  QCOMPARE(decls.at(0)->identifier().toString(), QString("i"));
+  QVERIFY(decls.at(0)->abstractType());
+  QCOMPARE(decls.at(0)->abstractType()->toString(), QString("float"));
+  QCOMPARE(decls.at(0)->range().start.line, 14);
+}
+
 void TestDUChain::testRValueReference() {
   //                                         1         2         3
   //                                123456789012345678901234567890
@@ -364,6 +429,16 @@ void TestDUChain::testDecltype()
   QVERIFY(x7Dec->abstractType()->equals(x6Dec->abstractType().constData()));
 }
 
+void TestDUChain::testDecltypeTypedef()
+{
+  LockedTopDUContext top = parse(readCodeFile("testDecltypeTypedef.cpp"), DumpNone);
+  QVERIFY(top);
+
+  QCOMPARE(top->localDeclarations().size(), 4);
+  QCOMPARE(TypeUtils::unAliasedType(top->localDeclarations().at(2)->abstractType())->toString(), QString("int"));
+  QCOMPARE(TypeUtils::unAliasedType(top->localDeclarations().at(3)->abstractType())->toString(), QString("int"));
+}
+
 void TestDUChain::testTrailingReturnType()
 {
   {
@@ -603,6 +678,86 @@ void TestDUChain::testLambdaCapture()
   QCOMPARE(funType->returnType().cast<IntegralType>()->dataType(), (uint) IntegralType::TypeVoid);
 }
 
+void TestDUChain::testTemplateSpecializeVolatile()
+{
+  // see also: https://bugs.kde.org/show_bug.cgi?id=321885
+  const QByteArray code(
+    "template<typename _Tp>\n"
+    "struct numeric_limits { };\n"
+    "\n"
+    "template<typename _Tp>\n"
+    "  struct numeric_limits<const _Tp>\n"
+    "  : public numeric_limits<_Tp> { };\n"
+    "\n"
+    "template<typename _Tp>\n"
+    "  struct numeric_limits<volatile _Tp>\n"
+    "  : public numeric_limits<_Tp> { };\n"
+    "\n"
+    "template<typename _Tp>\n"
+    "  struct numeric_limits<const volatile _Tp>\n"
+    "  : public numeric_limits<_Tp> { };\n"
+    "\n"
+    "template<>\n"
+    "struct numeric_limits<double>\n"
+    "{\n"
+    "  static double epsilon();\n"
+    "};\n"
+    "\n"
+    "int main() {\n"
+    "  numeric_limits< double > a;\n"
+    "  a.epsilon();\n"
+    "  numeric_limits< const double > b;\n"
+    "  b.epsilon();\n"
+    "  numeric_limits< volatile double > c;\n"
+    "  c.epsilon();\n"
+    "  numeric_limits< const volatile double > d;\n"
+    "  d.epsilon();\n"
+    "}\n"
+  );
+  LockedTopDUContext top = parse(code, DumpAll);
+  QVERIFY(top);
+  DUChainReadLocker lock;
+  QVERIFY(top->problems().isEmpty());
+
+  QCOMPARE(top->localDeclarations().size(), 6);
+
+  TemplateDeclaration* tplDec = dynamic_cast<TemplateDeclaration*>(top->localDeclarations().at(0));
+  QVERIFY(tplDec);
+  QCOMPARE(tplDec->specializationsSize(), 4u);
+  QCOMPARE(tplDec->instantiations().size(), 7);
+
+  int declIdx = 1;
+  foreach(const QString id, QStringList() << "const _Tp" << "volatile _Tp" << "const volatile _Tp") {
+    Declaration* specRaw = top->localDeclarations().at(declIdx++);
+    TemplateDeclaration* spec = dynamic_cast<TemplateDeclaration*>(specRaw);
+    QVERIFY(spec);
+    QVERIFY(spec->specializedFrom().isValid());
+    QVERIFY(spec->specializedWith().isValid());
+    InstantiationInformation info = spec->specializedWith().information();
+    QCOMPARE(info.templateParametersSize(), 1u);
+    QVERIFY(info.templateParameters()[0].isValid());
+    AbstractType::Ptr specParam = info.templateParameters()[0].abstractType();
+    QVERIFY(specParam);
+    QCOMPARE(specParam->toString(), id);
+  }
+
+  QCOMPARE(top->childContexts().last()->localDeclarations().size(), 4);
+  QCOMPARE(top->childContexts().last()->localDeclarations().at(0)->abstractType()->toString(),
+           QString("numeric_limits< double >"));
+  QVERIFY(top->childContexts().last()->findUseAt(CursorInRevision(23, 6)) != -1);
+  QEXPECT_FAIL("", "Delegation of the const/volatile to the non-const/volatile does not work", Abort);
+  QCOMPARE(top->childContexts().last()->localDeclarations().at(1)->abstractType()->toString(),
+           QString("numeric_limits< const double >"));
+  QCOMPARE(top->childContexts().last()->localDeclarations().at(2)->abstractType()->toString(),
+           QString("numeric_limits< volatile double >"));
+  QCOMPARE(top->childContexts().last()->localDeclarations().at(3)->abstractType()->toString(),
+           QString("numeric_limits< const volatile double >"));
+  // should all work
+  QVERIFY(top->childContexts().last()->findUseAt(CursorInRevision(25, 6)) != -1);
+  QVERIFY(top->childContexts().last()->findUseAt(CursorInRevision(27, 6)) != -1);
+  QVERIFY(top->childContexts().last()->findUseAt(CursorInRevision(29, 6)) != -1);
+}
+
 void TestDUChain::testTemplateSpecializeArray()
 {
   // see also: https://bugs.kde.org/show_bug.cgi?id=294306
@@ -715,6 +870,7 @@ void TestDUChain::testTemplateSpecializeRValue()
   QVERIFY(specParam.cast<ReferenceType>());
   QVERIFY(specParam.cast<ReferenceType>()->isRValue());
   QCOMPARE(specParam->toString(), QString("T&&"));
+  QCOMPARE(Cpp::simplifiedTypeString(specParam, top), QString("T&&"));
 
   QCOMPARE(top->childContexts().last()->localDeclarations().size(), 2);
   QCOMPARE(top->childContexts().last()->localDeclarations().at(0)->abstractType()->toString(),
@@ -773,13 +929,28 @@ void TestDUChain::testAuto()
     "const auto a4 = i;\n"
     "const auto& a5 = i;\n"
     "const auto&& a6 = i;\n"
+    "auto a7(1.1d);\n"
+    "auto a8(i);\n"
+    "auto a9 /** test **/ (i);\n"
+    "auto a10(a2);\n"
+    "auto a11(a3);\n"
+    "auto a12(a4);\n"
+    "auto a13(a5);\n"
+    "auto a14(a6);\n"
+    "void f() {\n"
+    "  if (auto c1 = 0.0) {}\n"
+    "}\n"
+    "struct FOO {};\n"
+    "auto a15 = FOO{};\n"
+    "auto a16 = FOO{1};\n"
+
   );
   LockedTopDUContext top = parse(code, DumpAll);
   QVERIFY(top);
   DUChainReadLocker lock;
   QVERIFY(top->problems().isEmpty());
 
-  QCOMPARE(top->localDeclarations().count(), 7);
+  QCOMPARE(top->localDeclarations().count(), 19);
 
   Declaration* dec = top->localDeclarations().at(1);
   QVERIFY(dec->type<IntegralType>());
@@ -815,6 +986,30 @@ void TestDUChain::testAuto()
   QVERIFY(dec->type<ReferenceType>()->isRValue());
   QVERIFY(dec->type<ReferenceType>()->baseType().cast<IntegralType>());
   QCOMPARE(dec->type<ReferenceType>()->baseType().cast<IntegralType>()->dataType(), (uint) IntegralType::TypeChar);
+
+  dec = top->localDeclarations().at(7);
+  QVERIFY(dec->type<IntegralType>());
+  QCOMPARE(dec->type<IntegralType>()->dataType(), (uint) IntegralType::TypeDouble);
+
+  for (int i = 8; i < 15; ++i) {
+    dec = top->localDeclarations().at(i);
+    QVERIFY(dec->type<IntegralType>());
+    QCOMPARE(dec->type<IntegralType>()->dataType(), (uint) IntegralType::TypeChar);
+    QCOMPARE(dec->abstractType()->modifiers(), (quint64) AbstractType::NoModifiers);
+  }
+
+  dec = top->localDeclarations().at(15)->internalContext()->childContexts().first()->findDeclarations(Identifier("c1")).first();
+  QVERIFY(dec->type<IntegralType>());
+  QCOMPARE(dec->type<IntegralType>()->dataType(), (uint) IntegralType::TypeDouble);
+
+  StructureType::Ptr foo = top->localDeclarations().at(16)->type<StructureType>();
+  QVERIFY(foo);
+
+  dec = top->localDeclarations().at(17);
+  QVERIFY(dec->abstractType()->equals(foo.constData()));
+
+  dec = top->localDeclarations().at(18);
+  QVERIFY(dec->abstractType()->equals(foo.constData()));
 }
 
 void TestDUChain::testNoexcept()
@@ -830,3 +1025,24 @@ void TestDUChain::testNoexcept()
   DUChainReadLocker lock;
   QVERIFY(top->problems().isEmpty());
 }
+
+void TestDUChain::testInlineNamespace()
+{
+    const QByteArray code(
+      "inline namespace a { void foo (); }\n"
+      "int main() {\n"
+      "  a::foo();\n"
+      "  foo();\n"
+      "}\n"
+  );
+  LockedTopDUContext top = parse(code, DumpAll);
+  QVERIFY(top);
+  DUChainReadLocker lock;
+  QVERIFY(top->problems().isEmpty());
+
+  Declaration* foo = top->childContexts().first()->localDeclarations().first();
+  QCOMPARE(foo->uses().size(), 1);
+  QCOMPARE(foo->uses().begin()->size(), 2);
+  QCOMPARE(foo->uses().begin()->at(0), RangeInRevision(2, 5, 2, 8));
+  QCOMPARE(foo->uses().begin()->at(1), RangeInRevision(3, 2, 3, 5));
+}
