@@ -17,6 +17,7 @@
 */
 
 #include "test_expressionparser.h"
+#include "testhelper.h"
 
 #include <QtTest/QtTest>
 
@@ -47,6 +48,7 @@
 
 #include <tests/autotestshell.h>
 #include <tests/testcore.h>
+#include <tests/testhelpers.h>
 
 #include <typeinfo>
 
@@ -78,62 +80,6 @@ char* debugString( const QString& str ) {
   return ret;
 }
 
-namespace QTest {
-  template<>
-  char* toString(const Cursor& cursor)
-  {
-    QByteArray ba = "Cursor(";
-    ba += QByteArray::number(cursor.line()) + ", " + QByteArray::number(cursor.column());
-    ba += ')';
-    return qstrdup(ba.data());
-  }
-  template<>
-  char* toString(const QualifiedIdentifier& id)
-  {
-    QByteArray arr = id.toString().toLatin1();
-    return qstrdup(arr.data());
-  }
-  template<>
-  char* toString(const Identifier& id)
-  {
-    QByteArray arr = id.toString().toLatin1();
-    return qstrdup(arr.data());
-  }
-  /*template<>
-  char* toString(QualifiedIdentifier::MatchTypes t)
-  {
-    QString ret;
-    switch (t) {
-      case QualifiedIdentifier::NoMatch:
-        ret = "No Match";
-        break;
-      case QualifiedIdentifier::Contains:
-        ret = "Contains";
-        break;
-      case QualifiedIdentifier::ContainedBy:
-        ret = "Contained By";
-        break;
-      case QualifiedIdentifier::ExactMatch:
-        ret = "Exact Match";
-        break;
-    }
-    QByteArray arr = ret.toString().toLatin1();
-    return qstrdup(arr.data());
-  }*/
-  template<>
-  char* toString(const Declaration& def)
-  {
-    QString s = QString("Declaration %1 (%2): %3").arg(def.identifier().toString()).arg(def.qualifiedIdentifier().toString()).arg(reinterpret_cast<long>(&def));
-    return qstrdup(s.toLatin1().constData());
-  }
-  template<>
-  char* toString(const TypePtr<AbstractType>& type)
-  {
-    QString s = QString("Type: %1").arg(type ? type->toString() : QString("<null>"));
-    return qstrdup(s.toLatin1().constData());
-  }
-}
-
 #define TEST_FILE_PARSE_ONLY if (testFileParseOnly) QSKIP("Skip", SkipSingle);
 TestExpressionParser::TestExpressionParser()
 {
@@ -142,7 +88,7 @@ TestExpressionParser::TestExpressionParser()
 
 void TestExpressionParser::initTestCase()
 {
-  AutoTestShell::init();
+  AutoTestShell::init(QStringList() << "kdevcppsupport");
   TestCore* core = new TestCore();
   core->initialize(KDevelop::Core::NoUi);
   EnvironmentManager::init();
@@ -152,6 +98,7 @@ void TestExpressionParser::initTestCase()
 
 void TestExpressionParser::cleanupTestCase()
 {
+    TestCore::shutdown();
 }
 
 Declaration* TestExpressionParser::findDeclaration(DUContext* context, const Identifier& id, const CursorInRevision& position)
@@ -170,6 +117,56 @@ Declaration* TestExpressionParser::findDeclaration(DUContext* context, const Qua
   return 0;
 }
 
+void TestExpressionParser::testTemplateSpecialization()
+{
+  QByteArray method(//Boring class Foo has a function (myFoo) with a template argument
+                    "class Foo { template<class T> Foo& myFoo ( T& value ) { } };\n"
+                    //Boring class Foo defines a specialization of myFoo for use with ints
+                    "template<> Foo& Foo::myFoo <int> ( int& value ) { }\n"
+                    //Global function bar takes a template argument
+                    "template<class T> T& bar( T& value ) { }\n"
+                    //Global function bar has specialized definition for use with ints
+                    "template<> int& bar( int& value ) { }\n"
+                    //Class A takes two template arguments, and has a function (foo) that depends on them
+                    "template<class T, class U> class A { T foo(U value); };\n"
+                    //Class A defines its foo function as expected
+                    "template<class T, class U> T A<T,U>::foo(U value) {};\n"
+                    //Class A has a partially specialized form for when param U is an int
+                    "template<class T> class A<T, int> { T foo(int value); };\n"
+                    //Partially specialized class A defines its foo function as expected
+                    "template<class T> T A<T, int>::foo(int value) {};\n"
+                    //Class A also defines a fully specialized foo for params <char,int>
+                    "template<> char A<char, int>::foo(int value) {};\n");
+
+  DUContext* top = parse(method, DumpNone);
+
+  DUChainWriteLocker lock(DUChain::lock());
+
+  Declaration* defFooFooOpT = top->childContexts()[0]->localDeclarations()[0];
+  TemplateDeclaration* defFooFooOpint = dynamic_cast<TemplateDeclaration*>(top->localDeclarations()[1]);
+  QCOMPARE(defFooFooOpint->specializedFrom().data(), defFooFooOpT);
+
+  Declaration* defTBarT = top->localDeclarations()[2];
+  TemplateDeclaration* defintbarint = dynamic_cast<TemplateDeclaration*>(top->localDeclarations()[3]);
+  QCOMPARE(defintbarint->specializedFrom().data(), defTBarT);
+
+  Declaration* defTUA = top->localDeclarations()[4];
+  TemplateDeclaration* defTintA = dynamic_cast<TemplateDeclaration*>(top->localDeclarations()[6]);
+  QCOMPARE(defTintA->specializedFrom().data(), defTUA);
+
+  TemplateDeclaration* defTAfooU = dynamic_cast<TemplateDeclaration*>(top->localDeclarations()[5]);
+  QVERIFY(!defTAfooU->specializedFrom().data()); //Not a specialization
+
+  TemplateDeclaration* defTAfooint = dynamic_cast<TemplateDeclaration*>(top->localDeclarations()[7]);
+  QVERIFY(!defTAfooint->specializedFrom().data()); //Not a specialization
+
+  Declaration* decTAfooInt = dynamic_cast<Declaration*>(defTintA)->internalContext()->localDeclarations()[0];
+  TemplateDeclaration* defcharAfooint = dynamic_cast<TemplateDeclaration*>(top->localDeclarations()[8]);
+  QCOMPARE(defcharAfooint->specializedFrom().data(), decTAfooInt);
+
+  release(top);
+}
+
 void TestExpressionParser::testIntegralType() {
   DUContext* top = parse(" ", DumpNone);
   Cpp::ExpressionParser parser(true,true);
@@ -201,6 +198,15 @@ void TestExpressionParser::testIntegralType() {
     QCOMPARE(ciType->dataType(), (uint)ConstantIntegralType::TypeChar);
     QCOMPARE(ciType->value<char>(), 'x');
   }
+  {
+    Cpp::ExpressionEvaluationResult result = parser.evaluateType("char(1)", KDevelop::DUContextPointer(top));
+    QVERIFY(result.isValid());
+
+    AbstractType::Ptr aType(result.type.abstractType());
+    IntegralType::Ptr iType = aType.cast<IntegralType>();
+    QVERIFY(iType);
+    QCOMPARE(iType->dataType(), (uint)IntegralType::TypeChar);
+  }
   {
       Cpp::ExpressionEvaluationResult result = parser.evaluateType("5", KDevelop::DUContextPointer(top));
       QVERIFY(result.isValid());
@@ -481,6 +487,30 @@ void TestExpressionParser::testSmartPointer() {
   release(top);
 }
 
+void TestExpressionParser::testAutoTemplate()
+{
+  QByteArray code = "struct C { float m(); }; C c; template<class T> auto foo(T n) -> decltype(n.m()); template<class T> auto foo2(T n) -> decltype(n);";
+  LockedTopDUContext top = parse(code);
+  QVERIFY(top);
+  DUChainReadLocker lock;
+  QCOMPARE(top->localDeclarations().size(), 4);
+  FunctionType::Ptr fType = top->localDeclarations()[2]->abstractType().cast<FunctionType>();
+  QVERIFY(fType);
+  QVERIFY(fType->returnType());
+  DelayedType::Ptr ret = fType->returnType().cast<DelayedType>();
+  QVERIFY(ret);
+  QVERIFY(ret->kind() == DelayedType::Delayed);
+  Cpp::ExpressionParser parser;
+
+  ExpressionEvaluationResult res = parser.evaluateExpression("foo(c)", DUContextPointer(top));
+  QVERIFY(res.isValid());
+  QCOMPARE(res.type.abstractType()->toString(), QString("float"));
+
+  res = parser.evaluateExpression("foo2(c)", DUContextPointer(top));
+  QVERIFY(res.isValid());
+  QCOMPARE(res.type.abstractType()->toString(), QString("C"));
+}
+
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
@@ -507,6 +537,20 @@ void TestExpressionParser::testSimpleExpression() {
   Cpp::ExpressionParser parser;
   Cpp::ExpressionEvaluationResult result;
 
+  result = parser.evaluateType( "decltype(1+2)", KDevelop::DUContextPointer(testContext));
+  lock.lock();
+  QVERIFY(result.isValid());
+  QCOMPARE(result.type.abstractType()->toString(), QString("int"));
+  QVERIFY(!result.isInstance);
+  lock.unlock();
+  
+  result = parser.evaluateType( "decltype(c)()", KDevelop::DUContextPointer(testContext));
+  lock.lock();
+  QVERIFY(result.isValid());
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont"));
+  QVERIFY(!result.isInstance);
+  lock.unlock();
+
   result = parser.evaluateType( "const Cont", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
@@ -630,6 +674,13 @@ void TestExpressionParser::testSimpleExpression() {
   QVERIFY(result.isInstance);
   lock.unlock();
 
+  result = parser.evaluateExpression( "new Cont{}", KDevelop::DUContextPointer(testContext));
+  lock.lock();
+  QVERIFY(result.isValid());
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont*"));
+  QVERIFY(result.isInstance);
+  lock.unlock();
+
   result = parser.evaluateExpression( "5", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
@@ -1011,6 +1062,15 @@ void TestExpressionParser::testOperators() {
   QCOMPARE(result.type.abstractType()->toString(), QString("Cont"));
   lock.unlock();
 
+  //A simple test: Constructing a type should always result in the type, no matter whether there is a constructor.
+  result = parser.evaluateExpression( "Cont{5}", KDevelop::DUContextPointer(ctx));
+  lock.lock();
+  QVERIFY(result.isValid());
+  QVERIFY(result.isInstance);
+  QVERIFY(result.type.abstractType());
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont"));
+  lock.unlock();
+
   lock.lock();
   release(ctx);
 }
@@ -1107,6 +1167,270 @@ void TestExpressionParser::testTemplateFunctions() {
   }
 }
 
+void TestExpressionParser::testTypeID()
+{
+  QByteArray method("namespace std { struct type_info{ const char* name(); }; }\nvoid f() {float f;}");
+
+  DUContext* top = parse(method, DumpNone);
+
+  DUChainWriteLocker lock;
+  QCOMPARE(top->localDeclarations().count(), 2);
+  QCOMPARE(top->childContexts().size(), 3);
+  DUContextPointer fBody(top->childContexts().last());
+
+  Declaration* tiDec = top->findDeclarations(QualifiedIdentifier("::std::type_info")).first();
+
+  Cpp::ExpressionParser parser(false, true);
+
+  Cpp::ExpressionEvaluationResult result;
+
+  result = parser.evaluateExpression("typeid(int)", fBody);
+  QVERIFY(result.isValid());
+  QVERIFY(result.type.type<StructureType>());
+  QVERIFY(result.type.abstractType()->equals(tiDec->abstractType().constData()));
+
+  result = parser.evaluateExpression("typeid(5)", fBody);
+  QVERIFY(result.isValid());
+  QVERIFY(result.type.type<StructureType>());
+  QVERIFY(result.type.abstractType()->equals(tiDec->abstractType().constData()));
+
+  result = parser.evaluateExpression("typeid(f)", fBody);
+  QVERIFY(result.isValid());
+  QVERIFY(result.type.type<StructureType>());
+  QVERIFY(result.type.abstractType()->equals(tiDec->abstractType().constData()));
+
+  result = parser.evaluateExpression("typeid(f).name()", fBody);
+  QVERIFY(result.isValid());
+  QVERIFY(result.type.type<PointerType>());
+  QVERIFY(result.type.type<PointerType>()->baseType().cast<IntegralType>());
+  QCOMPARE(result.type.type<PointerType>()->baseType().cast<IntegralType>()->dataType(), (uint)IntegralType::TypeChar);
+}
+
+void TestExpressionParser::testConstness()
+{
+  QByteArray method("struct A { int x; }; const A* a = new A;");
+
+  DUContext* top = parse(method, DumpNone);
+
+  DUChainWriteLocker lock;
+  QCOMPARE(top->localDeclarations().count(), 2);
+  QCOMPARE(top->childContexts().count(), 1);
+
+  // "a" is a pointer type, which itself is not const, but it points to const data
+  Declaration* a = top->localDeclarations().last();
+  QVERIFY(!TypeUtils::isConstant(a->abstractType()));
+  QVERIFY(a->abstractType().cast<PointerType>());
+  QVERIFY(TypeUtils::isConstant(a->abstractType().cast<PointerType>()->baseType()));
+
+  Cpp::ExpressionParser parser(false, true, true);
+  Cpp::ExpressionEvaluationResult result = parser.evaluateExpression("a->x", DUContextPointer(top));
+
+  QVERIFY(result.isValid());
+  QVERIFY(result.type);
+  QVERIFY(TypeUtils::isConstant(result.type.abstractType()));
+}
+
+void TestExpressionParser::testConstnessOverload()
+{
+  QByteArray method("struct A { const A& foo() const; A foo(); };\n"
+                    "const A constA; A nonConstA;");
+
+  DUContext* top = parse(method, DumpNone);
+
+  DUChainWriteLocker lock;
+  QCOMPARE(top->localDeclarations().count(), 3);
+  QCOMPARE(top->childContexts().count(), 1);
+
+  Declaration* constA = top->localDeclarations().at(1);
+  QVERIFY(TypeUtils::isConstant(constA->abstractType()));
+
+  Declaration* nonConstA = top->localDeclarations().at(2);
+  QVERIFY(!TypeUtils::isConstant(nonConstA->abstractType()));
+
+  Cpp::ExpressionParser parser(false, true, true);
+
+  Cpp::ExpressionEvaluationResult result = parser.evaluateExpression("constA.foo()", DUContextPointer(top));
+  QVERIFY(result.isValid());
+  QVERIFY(result.type);
+  // should be the const& version
+  AbstractType::Ptr type = result.type.abstractType();
+  QVERIFY(!TypeUtils::isConstant(type));
+  QVERIFY(type.cast<ReferenceType>());
+  QVERIFY(TypeUtils::isConstant(type.cast<ReferenceType>()->baseType()));
+  QVERIFY(type.cast<ReferenceType>()->baseType().cast<StructureType>());
+
+  result = parser.evaluateExpression("nonConstA.foo()", DUContextPointer(top));
+  QVERIFY(result.isValid());
+  QVERIFY(result.type);
+  // should be plain a
+  type = result.type.abstractType();
+  QVERIFY(!TypeUtils::isConstant(type));
+  QVERIFY(!TypeUtils::isReferenceType(type));
+  QVERIFY(type.cast<StructureType>());
+}
+
+void TestExpressionParser::testConstnessOverloadSubscript()
+{
+  QByteArray method("struct A { const A& operator[](int i) const; A operator[](int i); };\n"
+                    "const A constA; A nonConstA;");
+
+  DUContext* top = parse(method, DumpNone);
+
+  DUChainWriteLocker lock;
+  QCOMPARE(top->localDeclarations().count(), 3);
+  QCOMPARE(top->childContexts().count(), 1);
+
+  Declaration* constA = top->localDeclarations().at(1);
+  QVERIFY(TypeUtils::isConstant(constA->abstractType()));
+
+  Declaration* nonConstA = top->localDeclarations().at(2);
+  QVERIFY(!TypeUtils::isConstant(nonConstA->abstractType()));
+
+  Cpp::ExpressionParser parser(false, true, true);
+
+  Cpp::ExpressionEvaluationResult result = parser.evaluateExpression("constA[0]", DUContextPointer(top));
+  QVERIFY(result.isValid());
+  QVERIFY(result.type);
+  // should be the const& version
+  AbstractType::Ptr type = result.type.abstractType();
+  QVERIFY(!TypeUtils::isConstant(type));
+  QVERIFY(type.cast<ReferenceType>());
+  QVERIFY(TypeUtils::isConstant(type.cast<ReferenceType>()->baseType()));
+  QVERIFY(type.cast<ReferenceType>()->baseType().cast<StructureType>());
+
+  result = parser.evaluateExpression("nonConstA[0]", DUContextPointer(top));
+  QVERIFY(result.isValid());
+  QVERIFY(result.type);
+  // should be plain a
+  type = result.type.abstractType();
+  QVERIFY(!TypeUtils::isConstant(type));
+  QVERIFY(!TypeUtils::isReferenceType(type));
+  QVERIFY(type.cast<StructureType>());
+}
+
+void TestExpressionParser::testReference()
+{
+  QByteArray method("");
+
+  KDevelop::DUContextPointer testContext(parse(method, DumpNone));
+  DUChainWriteLocker lock;
+  Cpp::ExpressionParser parser(false, true, true);
+  Cpp::ExpressionEvaluationResult result;
+
+  result = parser.evaluateType( "int&", testContext);
+  QVERIFY(result.isValid());
+  QCOMPARE(result.type.abstractType()->toString(), QString("int&"));
+  QVERIFY(!result.isInstance);
+  QVERIFY(result.type.type<ReferenceType>());
+  QVERIFY(!result.type.type<ReferenceType>()->isRValue());
+
+  result = parser.evaluateType( "int&&", testContext);
+  QVERIFY(result.isValid());
+  QCOMPARE(result.type.abstractType()->toString(), QString("int&&"));
+  QVERIFY(!result.isInstance);
+  QVERIFY(result.type.type<ReferenceType>());
+  QVERIFY(result.type.type<ReferenceType>()->isRValue());
+
+}
+
+
+void TestExpressionParser::testCharacterTypes_data()
+{
+  QTest::addColumn<QString>("code");
+  QTest::addColumn<bool>("isString");
+  QTest::addColumn<uint>("type");
+
+  // see also: http://www.open-std.org/jtc1/sc22/wg21/docs/papers/2007/n2249.html
+  QTest::newRow("char") << "'a'" << false << (uint) IntegralType::TypeChar;
+  QTest::newRow("wchar_t") << "L'a'" << false << (uint) IntegralType::TypeWchar_t;
+  QTest::newRow("char16_t") << "u'a'" << false << (uint) IntegralType::TypeChar16_t;
+  QTest::newRow("char32_t") << "U'a';" << false << (uint) IntegralType::TypeChar32_t;
+
+  QTest::newRow("char-str") << "\"a\";" << true << (uint) IntegralType::TypeChar;
+  QTest::newRow("wchar_t-str") << "L\"a\";" << true << (uint) IntegralType::TypeWchar_t;
+  QTest::newRow("char16_t-str") << "u\"a\";" << true << (uint) IntegralType::TypeChar16_t;
+  QTest::newRow("char32_t-str") << "U\"a\";" << true << (uint) IntegralType::TypeChar32_t;
+}
+
+void TestExpressionParser::testCharacterTypes()
+{
+  QFETCH(QString, code);
+  QFETCH(bool, isString);
+  QFETCH(uint, type);
+
+  DUContext* top = parse("", DumpNone);
+
+  DUChainWriteLocker lock;
+  Cpp::ExpressionParser parser(false, true);
+  Cpp::ExpressionEvaluationResult result = parser.evaluateExpression(code.toUtf8(), DUContextPointer(top));
+  QVERIFY(result.isValid());
+  QVERIFY(result.type);
+
+  IntegralType::Ptr integralType;
+
+  if (isString) {
+    QVERIFY(result.type.type<PointerType>());
+    integralType = result.type.type<PointerType>()->baseType().cast<IntegralType>();
+  } else {
+    integralType = result.type.type<IntegralType>();
+  }
+
+  QVERIFY(integralType);
+  QEXPECT_FAIL("wchar_t-str", "not yet supported, see ExpressionVisitor::visitStringLiteral", Abort);
+  QEXPECT_FAIL("char16_t-str", "not yet supported, see ExpressionVisitor::visitStringLiteral", Abort);
+  QEXPECT_FAIL("char32_t-str", "not yet supported, see ExpressionVisitor::visitStringLiteral", Abort);
+  QCOMPARE(integralType->dataType(), type);
+}
+
+void TestExpressionParser::benchEvaluateType()
+{
+  DUChainWriteLocker lock;
+  QFETCH(DUContextPointer, context);
+  QFETCH(QByteArray, type);
+
+  QVERIFY(context);
+
+  Cpp::ExpressionParser parser;
+  QBENCHMARK {
+    parser.evaluateType(type, context);
+  }
+}
+
+void TestExpressionParser::benchEvaluateType_data()
+{
+  QTest::addColumn<DUContextPointer>("context");
+  QTest::addColumn<QByteArray>("type");
+
+  DUChainWriteLocker lock;
+  DUContextPointer top(parse("namespace ns { struct A { int b; }; } template<class T, typename _T2> struct B { }; int main(int argc) {ns::A a;}", DumpNone));
+  QVERIFY(top->childContexts().size() == 5);
+
+  QTest::newRow("global-char") << top << QByteArray("char");
+  QTest::newRow("global-bool") << top << QByteArray("bool");
+  QTest::newRow("global-false") << top << QByteArray("false ");
+  QTest::newRow("global-true") << top << QByteArray("true ");
+  QTest::newRow("global-void") << top << QByteArray("void");
+  QTest::newRow("global-float") << top << QByteArray("float");
+  QTest::newRow("global-double") << top << QByteArray("double");
+  QTest::newRow("global-int") << top << QByteArray("int");
+  QTest::newRow("global-int-number") << top << QByteArray("1234");
+  QTest::newRow("global-long-number") << top << QByteArray("1234l");
+  QTest::newRow("global-long-long-number") << top << QByteArray("1234ll");
+  QTest::newRow("global-ns::A") << top << QByteArray("ns::A");
+  QTest::newRow("global-B") << top << QByteArray("B");
+
+  DUContextPointer tplCtx(top->childContexts().at(2));
+  QTest::newRow("tpl-T") << tplCtx << QByteArray("T");
+  QTest::newRow("tpl-_T2") << tplCtx << QByteArray("_T2");
+  QTest::newRow("tpl-ns::A") << tplCtx << QByteArray("ns::A");
+  QTest::newRow("tpl-B") << tplCtx << QByteArray("B");
+
+  DUContextPointer mainCtx(top->childContexts().last());
+  QTest::newRow("main-ns::A") << mainCtx << QByteArray("ns::A");
+  QTest::newRow("main-a.b") << mainCtx << QByteArray("a.b");
+  QTest::newRow("main-B") << mainCtx << QByteArray("B");
+}
+
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
@@ -1118,7 +1442,7 @@ void TestExpressionParser::release(DUContext* top)
   //delete top;
 }
 
-DUContext* TestExpressionParser::parse(const QByteArray& unit, DumpAreas dump)
+TopDUContext* TestExpressionParser::parse(const QByteArray& unit, DumpAreas dump)
 {
   if (dump)
     kDebug(9007) << "==== Beginning new test case...:" << endl << unit;
