@@ -33,8 +33,6 @@
 #include <QReadWriteLock>
 #include <kactioncollection.h>
 #include <kaction.h>
-#include <QExtensionFactory>
-#include <QtDesigner/QExtensionFactory>
 
 #include <kdebug.h>
 #include <kcomponentdata.h>
@@ -45,6 +43,7 @@
 #include <kio/netaccess.h>
 #include <ktexteditor/document.h>
 #include <ktexteditor/view.h>
+#include <KDesktopFile>
 #include <language/codecompletion/codecompletion.h>
 
 #include <interfaces/icore.h>
@@ -82,7 +81,6 @@
 #include "cppduchain/navigation/navigationwidget.h"
 #include "cppduchain/cppduchain.h"
 #include "codegen/codeassistant.h"
-#include "codegen/cppnewclass.h"
 #include <interfaces/foregroundlock.h>
 //#include "codegen/makeimplementationprivate.h"
 
@@ -91,8 +89,10 @@
 #include "quickopen.h"
 #include "cppdebughelper.h"
 #include "codegen/simplerefactoring.h"
+#include "codegen/cppclasshelper.h"
 #include "includepathcomputer.h"
 #include "codecompletion/missingincludemodel.h"
+
 //#include <valgrind/callgrind.h>
 
 
@@ -150,10 +150,19 @@ public:
 };
 #endif
 
+
+static QStringList mimeTypesList()
+{
+    KDesktopFile desktopFile("services", QString("kdevcppsupport.desktop"));
+    const KConfigGroup& desktopGroup = desktopFile.desktopGroup();
+    QString mimeTypesStr = desktopGroup.readEntry("X-KDevelop-SupportedMimeTypes", "");
+    return mimeTypesStr.split(QChar(','), QString::SkipEmptyParts);
+}
+
 CppLanguageSupport::CppLanguageSupport( QObject* parent, const QVariantList& /*args*/ )
     : KDevelop::IPlugin( KDevCppSupportFactory::componentData(), parent ),
       KDevelop::ILanguageSupport(),
-      m_standardMacros(0)
+      m_mimeTypes(mimeTypesList())
 {
     m_self = this;
 
@@ -181,17 +190,20 @@ CppLanguageSupport::CppLanguageSupport( QObject* parent, const QVariantList& /*a
 
     if( quickOpen )
         quickOpen->registerProvider( IncludeFileDataProvider::scopes(), QStringList(i18n("Files")), m_quickOpenDataProvider );
-    else
-        kWarning() << "Quickopen not found";
+    // else we are in NoUi mode (duchainify, unit tests, ...) and hence cannot find the Quickopen plugin
 
 #ifdef DEBUG_UI_LOCKUP
-    m_blockTester = new UIBlockTester(LOCKUP_INTERVAL);
+    new UIBlockTester(LOCKUP_INTERVAL, this);
 #endif
 
     m_assistant = new Cpp::StaticCodeAssistant;
+
+    foreach(QString mimeType, m_mimeTypes){
+        KDevelop::IBuddyDocumentFinder::addFinder(mimeType,this);
+    }
 }
 
-void CppLanguageSupport::createActionsForMainWindow (Sublime::MainWindow* window, QString& _xmlFile, KActionCollection& actions)
+void CppLanguageSupport::createActionsForMainWindow (Sublime::MainWindow* /*window*/, QString& _xmlFile, KActionCollection& actions)
 {
     _xmlFile = xmlFile();
 
@@ -200,10 +212,6 @@ void CppLanguageSupport::createActionsForMainWindow (Sublime::MainWindow* window
     switchDefinitionDeclaration->setShortcut( Qt::CTRL | Qt::SHIFT | Qt::Key_C );
     connect(switchDefinitionDeclaration, SIGNAL(triggered(bool)), this, SLOT(switchDefinitionDeclaration()));
 
-    KAction* newClassAction = actions.addAction("code_new_class");
-    newClassAction->setText( i18n("Create &New Class") );
-    connect(newClassAction, SIGNAL(triggered(bool)), this, SLOT(newClassWizard()));
-    
 //    KAction* pimplAction = actions->addAction("code_private_implementation");
 //    pimplAction->setText( i18n("Make Class Implementation Private") );
 //    pimplAction->setShortcut(Qt::ALT | Qt::META | Qt::Key_P);
@@ -257,7 +265,11 @@ void CppLanguageSupport::switchDefinitionDeclaration()
     {
       lock.unlock();
       kDebug(9007) << "Parsing switch-candidate before switching" << switchCandidate;
-      DUChain::self()->waitForUpdate(IndexedString(switchCandidate), TopDUContext::VisibleDeclarationsAndContexts);
+      ReferencedTopDUContext updatedContext = DUChain::self()->waitForUpdate(IndexedString(switchCandidate), TopDUContext::VisibleDeclarationsAndContexts);
+      if (!updatedContext) {
+        kDebug(9007) << "Failed to update document:" << switchCandidate;
+        return;
+      }
     }
   }
   
@@ -383,7 +395,7 @@ CppLanguageSupport::~CppLanguageSupport()
 
     delete m_quickOpenDataProvider;
 
-    // Remove any documents waiting to be parsed from the background paser.
+    // Remove any documents waiting to be parsed from the background parser.
     core()->languageController()->backgroundParser()->clear(this);
 
 
@@ -392,15 +404,19 @@ CppLanguageSupport::~CppLanguageSupport()
     delete m_blockTester;
 #endif
     delete m_assistant;
+
+    foreach(QString mimeType, m_mimeTypes){
+        KDevelop::IBuddyDocumentFinder::removeFinder(mimeType);
+    }
 }
 
 CppLanguageSupport* CppLanguageSupport::self() {
     return m_self;
 }
 
-KDevelop::ParseJob *CppLanguageSupport::createParseJob( const KUrl &url )
+KDevelop::ParseJob *CppLanguageSupport::createParseJob( const IndexedString &url )
 {
-    return new CPPParseJob( url );
+    return new CPPParseJob( url, this );
 }
 
 KDevelop::ICodeHighlighting *CppLanguageSupport::codeHighlighting() const
@@ -408,6 +424,12 @@ KDevelop::ICodeHighlighting *CppLanguageSupport::codeHighlighting() const
     return m_highlights;
 }
 
+ICreateClassHelper* CppLanguageSupport::createClassHelper() const
+{
+    return new CppClassHelper;
+}
+
+
 void CppLanguageSupport::findIncludePathsForJob(CPPParseJob* job)
 {
   IncludePathComputer* comp = new IncludePathComputer(KUrl(job->document().str()), job->preprocessorProblemsPointer());
@@ -430,7 +452,7 @@ TopDUContext* CppLanguageSupport::standardContext(const KUrl& url, bool proxyCon
   DUChainReadLocker lock(DUChain::lock());
   const ParsingEnvironment* env = PreprocessJob::standardEnvironment();
   KDevelop::TopDUContext* top;
-  top = KDevelop::DUChain::self()->chainForDocument(url, env, Cpp::EnvironmentManager::self()->isSimplifiedMatching());
+  top = KDevelop::DUChain::self()->chainForDocument(url, env, Cpp::EnvironmentManager::self()->isSimplifiedMatching() || proxyContext);
 
   if( !top ) {
     //kDebug(9007) << "Could not find perfectly matching version of " << url << " for completion";
@@ -445,7 +467,7 @@ TopDUContext* CppLanguageSupport::standardContext(const KUrl& url, bool proxyCon
 
   if(top && (top->parsingEnvironmentFile() && top->parsingEnvironmentFile()->isProxyContext()) && !proxyContext)
   {
-    top = contentContextFromProxyContext(top);
+    top = DUChainUtils::contentContextFromProxyContext(top);
     if(!top)
     {
       kDebug(9007) << "WARNING: Proxy-context had invalid content-context";
@@ -455,6 +477,124 @@ TopDUContext* CppLanguageSupport::standardContext(const KUrl& url, bool proxyCon
   return top;
 }
 
+/**
+ * Anonymous namespace for IBuddyDocumentFinder related functions.
+ */
+namespace {
+
+/**
+ * @returns all extensions which match the given @p mimeType.
+ */
+QSet<QString> getExtensionsByMimeType(QString mimeType)
+{
+    KMimeType::Ptr ptr = KMimeType::mimeType(mimeType);
+
+    if (!ptr) {
+      return QSet<QString>();
+    }
+
+    QSet<QString> extensions;
+    foreach(const QString& pattern, ptr->patterns()) {
+      if (pattern.startsWith("*.")) {
+        extensions << pattern.mid(2);
+      }
+    }
+
+    return extensions;
+}
+
+QSet<QString> getHeaderFileExtensions()
+{
+    return getExtensionsByMimeType("text/x-c++hdr") | getExtensionsByMimeType("text/x-chdr");
+}
+
+QSet<QString> getSourceFileExtensions()
+{
+  return getExtensionsByMimeType("text/x-c++src") | getExtensionsByMimeType("text/x-csrc");
+}
+
+enum FileType {
+  Unknown, ///< Doesn't belong to C++
+  Header,  ///< Is a header file
+  Source   ///< Is a C(++) file
+};
+
+/**
+ * Generates the base path (without extension) and the file type
+ * for the specified url.
+ *
+ * @returns pair of base path and file type which has been found for @p url.
+ */
+QPair<QString,FileType> basePathAndType(const KUrl& url)
+{
+    QString path = url.toLocalFile();
+    int idxSlash = path.lastIndexOf("/");
+    int idxDot = path.lastIndexOf(".");
+    FileType fileType = Unknown;
+    QString basePath;
+    if (idxSlash >= 0 && idxDot >= 0 && idxDot > idxSlash) {
+        basePath = path.left(idxDot);
+        if (idxDot + 1 < path.length()) {
+            QString extension = path.mid(idxDot + 1);
+            if (getHeaderFileExtensions().contains(extension)) {
+                fileType = Header;
+            } else if (getSourceFileExtensions().contains(extension)) {
+                fileType = Source;
+            }
+        }
+    } else {
+        basePath = path;
+    }
+
+    return qMakePair(basePath, fileType);
+}
+
+}
+
+
+/**
+ * Behavior: Considers the URLs as buddy documents if the base path (=without extension)
+ * is the same, and one extension starts with h/H and the other one with c/C.
+ * For example, foo.hpp and foo.C are buddies.
+ */
+bool CppLanguageSupport::areBuddies(const KUrl& url1, const KUrl& url2)
+{
+    QPair<QString, FileType> type1 = basePathAndType(url1);
+    QPair<QString, FileType> type2 = basePathAndType(url2);
+    return(type1.first == type2.first && ((type1.second == Header && type2.second == Source) ||
+                                          (type1.second == Source && type2.second == Header)));
+}
+
+/**
+ * Behavior: places foo.h* / foo.H* left of foo.c* / foo.C*
+ */
+bool CppLanguageSupport::buddyOrder(const KUrl& url1, const KUrl& url2)
+{
+    QPair<QString, FileType> type1 = basePathAndType(url1);
+    QPair<QString, FileType> type2 = basePathAndType(url2);
+    // Precondition is that the two URLs are buddies, so don't check it
+    return(type1.second == Header && type2.second == Source);
+}
+
+QVector< KUrl > CppLanguageSupport::getPotentialBuddies(const KUrl& url) const
+{
+    QPair<QString, FileType> type = basePathAndType(url);
+    // Don't do anything for types we don't know
+    if (type.second == Unknown) {
+      return QVector< KUrl >();
+    }
+
+    // Depending on the buddy's file type we either generate source extensions (for headers)
+    // or header extensions (for sources)
+    const QSet<QString>& extensions = ( type.second == Header ? getSourceFileExtensions() : getHeaderFileExtensions() );
+    QVector< KUrl > buddies;
+    foreach(const QString& extension, extensions) {
+      buddies.append(KUrl(type.first + "." + extension));
+    }
+
+    return buddies;
+}
+
 QPair<QPair<QString, SimpleRange>, QString> CppLanguageSupport::cursorIdentifier(const KUrl& url, const SimpleCursor& position) const {
   KDevelop::IDocument* doc = core()->documentController()->documentForUrl(url);
   if(!doc || !doc->textDocument() || !doc->textDocument()->activeView())
@@ -654,10 +794,17 @@ QWidget* CppLanguageSupport::specialLanguageObjectNavigationWidget(const KUrl& u
     //Check whether tail contains arguments
     QString tail = found.second.trimmed(); ///@todo make this better.
     if(tail.startsWith("(")) {
-      int i = findClose( tail, 0 );
-      if(i != -1) {
-        text += tail.left(i+1);
+      //properly support macro expansions when arguments contain newlines
+      int foundClosingBrace = findClose( tail, 0 );
+      KDevelop::IDocument* doc = core()->documentController()->documentForUrl(url);
+      if(doc && doc->textDocument() && doc->textDocument()->activeView() && foundClosingBrace < 0) {
+        const int lines = doc->textDocument()->lines();
+        for (int lineNumber = position.line + 1; foundClosingBrace < 0 && lineNumber < lines; lineNumber++) {
+          tail += doc->textDocument()->line(lineNumber).trimmed();
+          foundClosingBrace = findClose( tail, 0 );
+        }
       }
+      text += tail.left(foundClosingBrace + 1);
     }
 
     {
@@ -675,12 +822,6 @@ QWidget* CppLanguageSupport::specialLanguageObjectNavigationWidget(const KUrl& u
     return new Cpp::NavigationWidget(*m.second, preprocessedBody);
 }
 
-void CppLanguageSupport::newClassWizard()
-{
-  //TODO: Should give some hint on where it should be added
-  SimpleRefactoring::self().createNewClass(0);
-}
-
 UIBlockTester::UIBlockTesterThread::UIBlockTesterThread( UIBlockTester& parent ) : QThread(), m_parent( parent ), m_stop(false) {
 }
 
@@ -702,7 +843,11 @@ UIBlockTester::UIBlockTesterThread::UIBlockTesterThread( UIBlockTester& parent )
          m_stop = true;
  }
 
- UIBlockTester::UIBlockTester( uint milliseconds ) : m_thread( *this ), m_msecs( milliseconds ) {
+UIBlockTester::UIBlockTester( uint milliseconds, QObject* parent )
+  : QObject(parent)
+  , m_thread( *this )
+  , m_msecs( milliseconds )
+{
          m_timer = new QTimer( this );
          m_timer->start( milliseconds/10 );
          connect( m_timer, SIGNAL(timeout()), this, SLOT(timer()) );
