@@ -42,7 +42,7 @@ using namespace rpp;
 #define RETURN_ON_FAIL(x) if(!(x)) { ++input; kDebug(9007) << "Preprocessor: Condition not satisfied"; return; }
 
 pp::pp(Preprocessor* preprocessor)
-  : m_environment(new Environment(this))
+  : m_environment(new Environment)
   , expand(this, 0, true)
   , m_preprocessor(preprocessor)
   , nextToken(0)
@@ -114,6 +114,12 @@ void pp::handle_directive(uint directive, Stream& input, Stream& output)
   static const uint includeNextDirective = KDevelop::IndexedString("include_next").index();
 
   skip_blanks (input, output);
+  while (!input.atEnd() && input != '\n' && input == '/' && input.peekNextCharacter() == '*')
+  {
+    skip_comment_or_divop(input, output);
+    skip_blanks (input, output);
+  }
+
   if(!(directive == ifndefDirective)) {
     hadGuardCandidate = true; //Too late, the guard must be the first directive
   }
@@ -317,8 +323,6 @@ void pp::handle_define (Stream& input)
 
     skip_blanks (++input, devnull()); // skip '('
     uint formal = skip_identifier(input);
-    if (formal)
-      macro->formalsList().append( KDevelop::IndexedString::fromIndex(formal) );
 
     skip_blanks(input, devnull());
 
@@ -330,14 +334,15 @@ void pp::handle_define (Stream& input)
 
       } while (input == '.');
     }
+    if (formal && !macro->variadics)
+      macro->formalsList().append( KDevelop::IndexedString::fromIndex(formal) );
+
 
     while (!input.atEnd() && input == ',')
     {
       skip_blanks(++input, devnull());
 
       uint formal = skip_identifier(input);
-      if (formal)
-        macro->formalsList().append( KDevelop::IndexedString::fromIndex(formal) );
 
       skip_blanks (input, devnull());
 
@@ -349,6 +354,9 @@ void pp::handle_define (Stream& input)
 
         } while (input == '.');
       }
+
+      if (formal && !macro->variadics)
+        macro->formalsList().append( KDevelop::IndexedString::fromIndex(formal) );
     }
 
     RETURN_ON_FAIL(input == ')');
@@ -1156,12 +1164,16 @@ int pp::next_token (Stream& input)
       }
       else if (isNumber(ch))
       {
+        KDevelop::SimpleCursor numericBeginPosition = input.inputPosition().castToSimpleCursor();
+
         PreprocessedContents byteNumber;
         {
           Stream ns(&byteNumber);
           skip_number(input, ns);
         }
 
+        KDevelop::SimpleCursor numericEndPosition = input.inputPosition().castToSimpleCursor();
+
         QString number(QString::fromUtf8(stringFromContents(byteNumber)));
         int base = 10;
         if (number.startsWith("0x")) {
@@ -1170,22 +1182,44 @@ int pp::next_token (Stream& input)
           base = 8;
         }
 
-        if (number.endsWith("lu", Qt::CaseInsensitive) || number.endsWith("ul", Qt::CaseInsensitive)) {
-          number.chop(2);
-          token_uvalue = number.toULong(0, base);
-          nextToken = TOKEN_UNUMBER;
-        } else if (number.endsWith('l', Qt::CaseInsensitive)) {
-          number.chop(1);
-          token_value = number.toLong(0, base);
-          nextToken = TOKEN_NUMBER;
-        } else if (number.endsWith('u', Qt::CaseInsensitive)) {
-          number.chop(1);
-          token_uvalue = number.toULong(0, base);
+        uint countUnsigned = 0, countLong = 0, countLongLong = 0;
+        while (true) {
+          if (number.endsWith("u", Qt::CaseInsensitive)) {
+                  ++countUnsigned;
+            number.chop(1);
+          } else if (number.endsWith("ll", Qt::CaseInsensitive)) { // "ll" before "l"
+                  ++countLongLong;
+            number.chop(2);
+          } else if (number.endsWith("l", Qt::CaseInsensitive)) {
+                  ++countLong;
+            number.chop (1);
+          } else {
+            break;
+          }
+        }
+
+        if ((countUnsigned > 1) || ((countLong + countLongLong) > 1)) {
+          KDevelop::ProblemPointer problem(new KDevelop::Problem);
+          problem->setFinalLocation(KDevelop::DocumentRange(currentFileName(), KDevelop::SimpleRange(numericBeginPosition, numericEndPosition)));
+          problem->setDescription(i18n("Invalid suffix combination"));
+          problemEncountered(problem);
+        }
+
+        bool parsedOk = 0;
+        if (countUnsigned) {
+          token_uvalue = number.toULongLong (&parsedOk, base);
           nextToken = TOKEN_UNUMBER;
         } else {
-          token_value = number.toLong(0, base);
+          token_value = number.toLongLong (&parsedOk, base);
           nextToken = TOKEN_NUMBER;
         }
+
+        if (!parsedOk) {
+          KDevelop::ProblemPointer problem(new KDevelop::Problem);
+          problem->setFinalLocation(KDevelop::DocumentRange(currentFileName(), KDevelop::SimpleRange(numericBeginPosition, numericEndPosition)));
+          problem->setDescription(i18n("Invalid numeric value"));
+          problemEncountered(problem);
+        }
       }
       else
       {
