@@ -156,6 +156,8 @@ private slots:
   void testNoQuadrupleColon();
   void testLookaheadMatches_data();
   void testLookaheadMatches();
+  void testMemberAccessInstance();
+  void testNestedInlineNamespace();
 public:
   enum DumpArea {
     DumpNone = 0,
