@@ -35,6 +35,7 @@
 #include <interfaces/icore.h>
 #include <interfaces/idocumentationcontroller.h>
 #include "astfactory.h"
+#include <cmakeduchaintypes.h>
 #include "cmakeutils.h"
 #include "icmakedocumentation.h"
 
@@ -115,8 +116,15 @@ void CMakeCodeCompletionModel::completionInvoked(View* view, const Range& range,
         }
         QDir dir(path);
         
-        m_paths=dir.entryList(QStringList() << tocomplete.mid(lastdir+1)+'*',
-                                          QDir::AllEntries | QDir::NoDotAndDotDot);
+        QFileInfoList paths=dir.entryInfoList(QStringList() << tocomplete.mid(lastdir+1)+'*',
+                                              QDir::AllEntries | QDir::NoDotAndDotDot);
+        m_paths.clear();
+        foreach(const QFileInfo& f, paths) {
+            QString currentPath = f.fileName();
+            if(f.isDir())
+                currentPath+='/';
+            m_paths += currentPath;
+        }
         
         numRows += m_paths.count();
     } else
@@ -144,9 +152,14 @@ CMakeCodeCompletionModel::Type CMakeCodeCompletionModel::indexType(int row) cons
 {
     if(m_inside)
     {
-        if(row<m_declarations.count())
-            return Variable;
-        else
+        if(row < m_declarations.count()) {
+            KDevelop::DUChainReadLocker lock;
+            Declaration* dec = m_declarations.at(row).declaration();
+            if (dec && dec->type<TargetType>())
+                return Target;
+            else
+                return Variable;
+        } else
             return Path;
     }
     else
@@ -166,19 +179,20 @@ QVariant CMakeCodeCompletionModel::data (const QModelIndex & index, int role) co
 
     if(role==Qt::DisplayRole && index.column()==CodeCompletionModel::Name)
     {
-        if(type==Command)
-            return s_commands[index.row()-m_declarations.size()];
-        else if(type==Path)
-            return m_paths[index.row()-m_declarations.size()];
-        else if(type==Variable || type==Macro)
-        {
-            int pos = index.row();
-            DUChainReadLocker lock(DUChain::lock());
-            Declaration* dec=m_declarations[pos].data();
-            if(dec)
-                return dec->identifier().toString();
-            else
-                return i18n("INVALID");
+        int pos = index.row();
+        switch(type) {
+            case Command:
+                return s_commands[pos-m_declarations.size()];
+            case Path:
+                return m_paths[pos-m_declarations.size()];
+            case Target:
+            case Variable:
+            case Macro: {
+                DUChainReadLocker lock(DUChain::lock());
+                Declaration* dec=m_declarations[pos].data();
+                
+                return dec ? dec->identifier().toString() : i18n("INVALID");
+            }
         }
     }
     else if(role==Qt::DisplayRole && index.column()==CodeCompletionModel::Prefix)
@@ -189,6 +203,7 @@ QVariant CMakeCodeCompletionModel::data (const QModelIndex & index, int role) co
             case Variable:  return i18n("Variable");
             case Macro:     return i18n("Macro");
             case Path:      return i18n("Path");
+            case Target:      return i18n("Target");
         }
     }
     else if(role==Qt::DecorationRole && index.column()==CodeCompletionModel::Icon)
@@ -198,6 +213,7 @@ QVariant CMakeCodeCompletionModel::data (const QModelIndex & index, int role) co
             case Command:   return KIcon("code-block");
             case Variable:  return KIcon("code-variable");
             case Macro:     return KIcon("code-function");
+            case Target:    return KIcon("system-run");
             case Path: {
                 QString url = m_paths[index.row()-m_declarations.size()];
                 return KIcon(KMimeType::findByUrl(url, 0, false, true)->iconName(url));
@@ -210,6 +226,7 @@ QVariant CMakeCodeCompletionModel::data (const QModelIndex & index, int role) co
             case Variable:
             case Command:
             case Path:
+            case Target:
                 break;
             case Macro:
             {
@@ -228,9 +245,9 @@ QVariant CMakeCodeCompletionModel::data (const QModelIndex & index, int role) co
                 foreach(const AbstractType::Ptr& t, func->arguments())
                 {
                     DelayedType::Ptr delay = t.cast<DelayedType>();
-                    args.append(delay->identifier().toString());
+                    args.append(delay ? delay->identifier().toString() : i18n("wrong"));
                 }
-                return '('+args.join(", ")+')';
+                return QString('('+args.join(", ")+')');
             }
         }
         
@@ -268,6 +285,9 @@ void CMakeCodeCompletionModel::executeCompletionItem(Document* document, const R
             
             document->replaceText(r, code);
         }   break;
+        case Target:
+            document->replaceText(word, data(index(row, Name, QModelIndex())).toString());
+            break;
     }
 }
 
