@@ -279,17 +279,6 @@ int CMakeProjectVisitor::visit(const CMakeAst *ast)
     return 1;
 }
 
-static QHash<QString, Target>::iterator findTargetForExecutable(const QString& exe, QHash<QString, Target>& targets)
-{
-    QHash<QString, Target>::iterator ret = targets.find(exe);
-    if(ret==targets.constEnd()) {
-        QString exe2 = exe;
-        exe2 = exe2.mid(exe2.indexOf('/')+1);
-        ret = targets.find(exe2);
-    }
-    return ret;
-}
-
 int CMakeProjectVisitor::visit( const AddTestAst * test)
 {
     Test t;
@@ -1464,6 +1453,11 @@ int CMakeProjectVisitor::visit(const ExecProgramAst *exec)
 
     foreach(const QString& arg, argsTemp)
     {
+        if(arg.contains("#[bin_dir]")) {
+            if(!exec->outputVariable().isEmpty())
+                m_vars->insert(exec->outputVariable(), QStringList("OFF"));
+            return 1;
+        }
         args += arg.split(' ');
     }
     kDebug(9042) << "Executing:" << execName << "::" << args << "in" << exec->workingDirectory();
@@ -1508,7 +1502,17 @@ int CMakeProjectVisitor::visit(const ExecuteProcessAst *exec)
             kDebug(9032) << "Error: trying to execute empty command";
             break;
         }
-        
+        else
+        {
+            foreach(const QString& arg, _args) {
+                if(arg.contains("#[bin_dir]")) {
+                    if(!exec->outputVariable().isEmpty())
+                        m_vars->insert(exec->outputVariable(), QStringList("OFF"));
+                    return 1;
+                }
+            }
+        }
+
         QString workingDir = exec->workingDirectory();
         if(!QFile::exists(workingDir))
         {
@@ -2035,7 +2039,7 @@ int CMakeProjectVisitor::visit(const StringAst *sast)
             break;
         case StringAst::Substring:
         {
-            QString res=sast->input()[0];
+            QString res=sast->input().join(QString());
             res=res.mid(sast->begin(), sast->length());
             m_vars->insert(sast->outputVariable(), QStringList(res));
         }   break;
