@@ -21,7 +21,8 @@
 
 #include "cmakelistsparser.h"
 #include "variablemap.h"
-namespace KDevelop { class Declaration; }
+
+#include <language/duchain/indexeddeclaration.h>
 
 struct Macro
 {
@@ -33,7 +34,7 @@ struct Macro
 
 struct CacheEntry
 {
-    CacheEntry(const QString& v=QString(), const QString &d=QString()) : value(v), doc(d) {}
+    CacheEntry(const QString& value=QString(), const QString &doc=QString()) : value(value), doc(doc) {}
     QString value;
     QString doc;
 };
@@ -42,12 +43,14 @@ struct Target
 {
     typedef QMap<QString, QString> Properties;
     enum Type { Library, Executable, Custom };
-    KDevelop::Declaration* declaration;
+    KDevelop::IndexedDeclaration declaration;
     QStringList files;
     Type type;
     CMakeFunctionDesc desc;
     QString name;
+    QStringList libraries;
 };
+typedef Target CMakeTarget; //Workaround for namespacing. fix me!
 
 struct Subdirectory
 {
@@ -56,12 +59,29 @@ struct Subdirectory
     QString build_dir;
 };
 
-enum PropertyType { GlobalProperty, DirectoryProperty, TargetProperty, SourceProperty, TestProperty, VariableProperty };
-typedef QMap<QString, QMap<QString, QStringList> > CategoryType;
+struct Test
+{
+    Test() {}
+    QString name;
+    QString executable;
+    QStringList arguments;
+    QHash<QString, QString> properties;
+};
+
+Q_DECLARE_TYPEINFO(Test, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(Subdirectory, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(Target, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(CacheEntry, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(Macro, Q_MOVABLE_TYPE);
+
+enum PropertyType { GlobalProperty, DirectoryProperty, TargetProperty, SourceProperty, TestProperty, CacheProperty, VariableProperty };
+typedef QHash<QString, QMap<QString, QStringList> > CategoryType;
 typedef QMap<PropertyType, CategoryType > CMakeProperties;
 
 typedef QHash<QString, Macro> MacroMap;
 typedef QHash<QString, QString> CMakeDefinitions;
 typedef QHash<QString, CacheEntry> CacheValues;
 
+Q_DECLARE_METATYPE(QList<Test>)
+
 #endif
